@@ -2,12 +2,15 @@
 # NumpyArray class
 #
 import numpy as np
+import sympy
+from scipy.sparse import csr_matrix, issparse
+
 import pybamm
-from scipy.sparse import issparse, csr_matrix
 
 
 class Array(pybamm.Symbol):
-    """node in the expression tree that holds an tensor type variable
+    """
+    Node in the expression tree that holds an tensor type variable
     (e.g. :class:`numpy.array`)
 
     Parameters
@@ -36,13 +39,14 @@ class Array(pybamm.Symbol):
         auxiliary_domains=None,
         entries_string=None,
     ):
+        # if
         if isinstance(entries, list):
             entries = np.array(entries)
         if entries.ndim == 1:
             entries = entries[:, np.newaxis]
         if name is None:
             name = "Array of shape {!s}".format(entries.shape)
-        self._entries = entries
+        self._entries = entries.astype(float)
         # Use known entries string to avoid re-hashing, where possible
         self.entries_string = entries_string
         super().__init__(name, domain=domain, auxiliary_domains=auxiliary_domains)
@@ -53,12 +57,12 @@ class Array(pybamm.Symbol):
 
     @property
     def ndim(self):
-        """ returns the number of dimensions of the tensor"""
+        """returns the number of dimensions of the tensor."""
         return self._entries.ndim
 
     @property
     def shape(self):
-        """ returns the number of entries along each dimension"""
+        """returns the number of entries along each dimension."""
         return self._entries.shape
 
     @property
@@ -75,24 +79,29 @@ class Array(pybamm.Symbol):
         else:
             entries = self._entries
             if issparse(entries):
-                self._entries_string = str(entries.__dict__)
+                dct = entries.__dict__
+                self._entries_string = ["shape", str(dct["_shape"])]
+                for key in ["data", "indices", "indptr"]:
+                    self._entries_string += [key, dct[key].tobytes()]
+                self._entries_string = tuple(self._entries_string)
+                # self._entries_string = str(entries.__dict__)
             else:
-                self._entries_string = entries.tobytes()
+                self._entries_string = (entries.tobytes(),)
 
     def set_id(self):
-        """ See :meth:`pybamm.Symbol.set_id()`. """
+        """See :meth:`pybamm.Symbol.set_id()`."""
         self._id = hash(
-            (self.__class__, self.name, self.entries_string) + tuple(self.domain)
+            (self.__class__, self.name) + self.entries_string + tuple(self.domain)
         )
 
     def _jac(self, variable):
-        """ See :meth:`pybamm.Symbol._jac()`. """
+        """See :meth:`pybamm.Symbol._jac()`."""
         # Return zeros of correct size
         jac = csr_matrix((self.size, variable.evaluation_array.count(True)))
         return pybamm.Matrix(jac)
 
-    def new_copy(self):
-        """ See :meth:`pybamm.Symbol.new_copy()`. """
+    def create_copy(self):
+        """See :meth:`pybamm.Symbol.new_copy()`."""
         return self.__class__(
             self.entries,
             self.name,
@@ -102,9 +111,18 @@ class Array(pybamm.Symbol):
         )
 
     def _base_evaluate(self, t=None, y=None, y_dot=None, inputs=None):
-        """ See :meth:`pybamm.Symbol._base_evaluate()`. """
+        """See :meth:`pybamm.Symbol._base_evaluate()`."""
         return self._entries
 
+    def is_constant(self):
+        """See :meth:`pybamm.Symbol.is_constant()`."""
+        return True
+
+    def to_equation(self):
+        """Returns the value returned by the node when evaluated."""
+        entries_list = self.entries.tolist()
+        return sympy.Array(entries_list)
+
 
 def linspace(start, stop, num=50, **kwargs):
     """
