@@ -1,13 +1,16 @@
 #
 # IndependentVariable class
 #
+import sympy
+
 import pybamm
 
 KNOWN_COORD_SYS = ["cartesian", "spherical polar"]
 
 
 class IndependentVariable(pybamm.Symbol):
-    """A node in the expression tree representing an independent variable
+    """
+    A node in the expression tree representing an independent variable.
 
     Used for expressing functions depending on a spatial variable or time
 
@@ -25,18 +28,26 @@ class IndependentVariable(pybamm.Symbol):
         super().__init__(name, domain=domain, auxiliary_domains=auxiliary_domains)
 
     def _evaluate_for_shape(self):
-        """ See :meth:`pybamm.Symbol.evaluate_for_shape_using_domain()` """
+        """See :meth:`pybamm.Symbol.evaluate_for_shape_using_domain()`"""
         return pybamm.evaluate_for_shape_using_domain(
             self.domain, self.auxiliary_domains
         )
 
     def _jac(self, variable):
-        """ See :meth:`pybamm.Symbol._jac()`. """
+        """See :meth:`pybamm.Symbol._jac()`."""
         return pybamm.Scalar(0)
 
+    def to_equation(self):
+        """Convert the node and its subtree into a SymPy equation."""
+        if self.print_name is not None:
+            return sympy.Symbol(self.print_name)
+        else:
+            return sympy.Symbol(self.name)
+
 
 class Time(IndependentVariable):
-    """A node in the expression tree representing time
+    """
+    A node in the expression tree representing time.
 
     *Extends:* :class:`Symbol`
     """
@@ -44,12 +55,12 @@ class Time(IndependentVariable):
     def __init__(self):
         super().__init__("time")
 
-    def new_copy(self):
-        """ See :meth:`pybamm.Symbol.new_copy()`. """
+    def create_copy(self):
+        """See :meth:`pybamm.Symbol.new_copy()`."""
         return Time()
 
     def _base_evaluate(self, t=None, y=None, y_dot=None, inputs=None):
-        """ See :meth:`pybamm.Symbol._base_evaluate()`. """
+        """See :meth:`pybamm.Symbol._base_evaluate()`."""
         if t is None:
             raise ValueError("t must be provided")
         return t
@@ -61,9 +72,14 @@ class Time(IndependentVariable):
         """
         return 0
 
+    def to_equation(self):
+        """Convert the node and its subtree into a SymPy equation."""
+        return sympy.Symbol("t")
+
 
 class SpatialVariable(IndependentVariable):
-    """A node in the expression tree representing a spatial variable
+    """
+    A node in the expression tree representing a spatial variable.
 
     Parameters
     ----------
@@ -102,15 +118,16 @@ class SpatialVariable(IndependentVariable):
                 "domain cannot be particle if name is '{}'".format(name)
             )
 
-    def new_copy(self):
-        """ See :meth:`pybamm.Symbol.new_copy()`. """
+    def create_copy(self):
+        """See :meth:`pybamm.Symbol.new_copy()`."""
         return self.__class__(
             self.name, self.domain, self.auxiliary_domains, self.coord_sys
         )
 
 
 class SpatialVariableEdge(SpatialVariable):
-    """A node in the expression tree representing a spatial variable, which evaluates
+    """
+    A node in the expression tree representing a spatial variable, which evaluates
     on the edges
 
     Parameters
@@ -127,7 +144,7 @@ class SpatialVariableEdge(SpatialVariable):
     def __init__(self, name, domain=None, auxiliary_domains=None, coord_sys=None):
         super().__init__(name, domain, auxiliary_domains, coord_sys)
 
-    def evaluates_on_edges(self, dimension):
+    def _evaluates_on_edges(self, dimension):
         return True
 
 
