@@ -4,7 +4,7 @@
 import pybamm
 import casadi
 import numpy as np
-from scipy.interpolate import PchipInterpolator, CubicSpline
+from scipy import special
 
 
 class CasadiConverter(object):
@@ -47,7 +47,7 @@ class CasadiConverter(object):
             return casadi_symbol
 
     def _convert(self, symbol, t, y, y_dot, inputs):
-        """ See :meth:`CasadiConverter.convert()`. """
+        """See :meth:`CasadiConverter.convert()`."""
         if isinstance(
             symbol,
             (
@@ -76,6 +76,8 @@ class CasadiConverter(object):
             converted_left = self.convert(left, t, y, y_dot, inputs)
             converted_right = self.convert(right, t, y, y_dot, inputs)
 
+            if isinstance(symbol, pybamm.Modulo):
+                return casadi.fmod(converted_left, converted_right)
             if isinstance(symbol, pybamm.Minimum):
                 return casadi.fmin(converted_left, converted_right)
             if isinstance(symbol, pybamm.Maximum):
@@ -88,6 +90,10 @@ class CasadiConverter(object):
             converted_child = self.convert(symbol.child, t, y, y_dot, inputs)
             if isinstance(symbol, pybamm.AbsoluteValue):
                 return casadi.fabs(converted_child)
+            if isinstance(symbol, pybamm.Floor):
+                return casadi.floor(converted_child)
+            if isinstance(symbol, pybamm.Ceiling):
+                return casadi.ceil(converted_child)
             return symbol._unary_evaluate(converted_child)
 
         elif isinstance(symbol, pybamm.Function):
@@ -123,8 +129,21 @@ class CasadiConverter(object):
                 return casadi.log(*converted_children)
             elif symbol.function == np.sign:
                 return casadi.sign(*converted_children)
-            elif isinstance(symbol.function, (PchipInterpolator, CubicSpline)):
-                return casadi.interpolant("LUT", "bspline", [symbol.x], symbol.y)(
+            elif symbol.function == special.erf:
+                return casadi.erf(*converted_children)
+            elif isinstance(symbol, pybamm.Interpolant):
+                if symbol.interpolator == "linear":
+                    solver = "linear"
+                elif symbol.interpolator == "cubic spline":
+                    solver = "bspline"
+                elif symbol.interpolator == "pchip":
+                    raise NotImplementedError(
+                        "The interpolator 'pchip' is not supported by CasAdi. "
+                        "Use 'linear' or 'cubic spline' instead. "
+                        "Alternatively, set 'model.convert_to_format = 'python'' "
+                        "and use a non-CasADi solver. "
+                    )
+                return casadi.interpolant("LUT", solver, symbol.x, symbol.y.flatten())(
                     *converted_children
                 )
             elif symbol.function.__name__.startswith("elementwise_grad_of_"):
