@@ -1,13 +1,17 @@
 #
 # Base Symbol Class for the expression tree
 #
-import pybamm
+import copy
+import numbers
 
 import anytree
-import numbers
-import copy
 import numpy as np
+import sympy
 from anytree.exporter import DotExporter
+from scipy.sparse import csr_matrix, issparse
+
+import pybamm
+from pybamm.expression_tree.printing.print_name import prettify_print_name
 
 
 def domain_size(domain):
@@ -26,9 +30,11 @@ def domain_size(domain):
         "negative electrode": 11,
         "separator": 13,
         "positive electrode": 17,
+        "working electrode": 19,
+        "working particle": 23,
+        "negative particle size": 29,
+        "positive particle size": 31,
     }
-    if isinstance(domain, str):
-        domain = [domain]
     if domain in [[], None]:
         size = 1
     elif all(dom in fixed_domain_sizes for dom in domain):
@@ -39,7 +45,7 @@ def domain_size(domain):
 
 
 def create_object_of_size(size, typ="vector"):
-    "Return object, consisting of NaNs, of the right shape"
+    """Return object, consisting of NaNs, of the right shape."""
     if typ == "vector":
         return np.nan * np.ones((size, 1))
     elif typ == "matrix":
@@ -61,8 +67,121 @@ def evaluate_for_shape_using_domain(domain, auxiliary_domains=None, typ="vector"
     return create_object_of_size(_domain_size * _auxiliary_domain_sizes, typ)
 
 
+def is_constant(symbol):
+    return isinstance(symbol, numbers.Number) or symbol.is_constant()
+
+
+def is_scalar_x(expr, x):
+    """
+    Utility function to test if an expression evaluates to a constant scalar value
+    """
+    if is_constant(expr):
+        result = expr.evaluate_ignoring_errors(t=None)
+        return isinstance(result, numbers.Number) and result == x
+    else:
+        return False
+
+
+def is_scalar_zero(expr):
+    """
+    Utility function to test if an expression evaluates to a constant scalar zero
+    """
+    return is_scalar_x(expr, 0)
+
+
+def is_scalar_one(expr):
+    """
+    Utility function to test if an expression evaluates to a constant scalar one
+    """
+    return is_scalar_x(expr, 1)
+
+
+def is_scalar_minus_one(expr):
+    """
+    Utility function to test if an expression evaluates to a constant scalar minus one
+    """
+    return is_scalar_x(expr, -1)
+
+
+def is_matrix_x(expr, x):
+    """
+    Utility function to test if an expression evaluates to a constant matrix value
+    """
+    if isinstance(expr, pybamm.Broadcast):
+        return is_scalar_x(expr.child, x) or is_matrix_x(expr.child, x)
+
+    if is_constant(expr):
+        result = expr.evaluate_ignoring_errors(t=None)
+        return (
+            issparse(result)
+            and (
+                (x == 0 and np.prod(len(result.__dict__["data"])) == 0)
+                or (
+                    len(result.__dict__["data"]) == np.prod(result.shape)
+                    and np.all(result.__dict__["data"] == x)
+                )
+            )
+        ) or (isinstance(result, np.ndarray) and np.all(result == x))
+    else:
+        return False
+
+
+def is_matrix_zero(expr):
+    """
+    Utility function to test if an expression evaluates to a constant matrix zero
+    """
+    return is_matrix_x(expr, 0)
+
+
+def is_matrix_one(expr):
+    """
+    Utility function to test if an expression evaluates to a constant matrix one
+    """
+    return is_matrix_x(expr, 1)
+
+
+def is_matrix_minus_one(expr):
+    """
+    Utility function to test if an expression evaluates to a constant matrix minus one
+    """
+    return is_matrix_x(expr, -1)
+
+
+def simplify_if_constant(symbol):
+    """
+    Utility function to simplify an expression tree if it evalutes to a constant
+    scalar, vector or matrix
+    """
+    domain = symbol.domain
+    auxiliary_domains = symbol.auxiliary_domains
+    if symbol.is_constant():
+        result = symbol.evaluate_ignoring_errors()
+        if result is not None:
+            if (
+                isinstance(result, numbers.Number)
+                or (isinstance(result, np.ndarray) and result.ndim == 0)
+                or isinstance(result, np.bool_)
+            ):
+                return pybamm.Scalar(result)
+            elif isinstance(result, np.ndarray) or issparse(result):
+                if result.ndim == 1 or result.shape[1] == 1:
+                    return pybamm.Vector(
+                        result, domain=domain, auxiliary_domains=auxiliary_domains
+                    )
+                else:
+                    # Turn matrix of zeros into sparse matrix
+                    if isinstance(result, np.ndarray) and np.all(result == 0):
+                        result = csr_matrix(result)
+                    return pybamm.Matrix(
+                        result, domain=domain, auxiliary_domains=auxiliary_domains
+                    )
+
+    return symbol
+
+
 class Symbol(anytree.NodeMixin):
-    """Base node class for the expression tree
+    """
+    Base node class for the expression tree.
 
     Parameters
     ----------
@@ -76,13 +195,12 @@ class Symbol(anytree.NodeMixin):
         is valid over all domains)
     auxiliary_domains : dict of str
         dictionary of auxiliary domains over which the node is valid (empty dictionary
-        indicates no auxiliary domains). Keys can be "secondary" or "tertiary". The
-        symbol is broadcast over its auxiliary domains.
+        indicates no auxiliary domains). Keys can be "secondary", "tertiary" or
+        "quaternary". The symbol is broadcast over its auxiliary domains.
         For example, a symbol might have domain "negative particle", secondary domain
         "separator" and tertiary domain "current collector" (`domain="negative
         particle", auxiliary_domains={"secondary": "separator", "tertiary": "current
         collector"}`).
-
     """
 
     def __init__(self, name, children=None, domain=None, auxiliary_domains=None):
@@ -92,6 +210,11 @@ class Symbol(anytree.NodeMixin):
         if children is None:
             children = []
 
+        # Store "orphans", which are separate from children as they do not have a
+        # parent node, so they do not cause tree corruption errors when used again
+        # in a different part of the tree
+        self._orphans = children
+
         for child in children:
             # copy child before adding
             # this also adds copy.copy(child) to self.children
@@ -106,6 +229,9 @@ class Symbol(anytree.NodeMixin):
         # Set domain (and hence id)
         self.domain = domain
 
+        self._saved_evaluates_on_edges = {}
+        self._print_name = None
+
         # Test shape on everything but nodes that contain the base Symbol class or
         # the base BinaryOperator class
         if pybamm.settings.debug_mode is True:
@@ -123,13 +249,12 @@ class Symbol(anytree.NodeMixin):
 
         Note: it is assumed that children of a node are not modified after initial
         creation
-
         """
         return self.cached_children
 
     @property
     def name(self):
-        """name of the node"""
+        """name of the node."""
         return self._name
 
     @name.setter
@@ -143,7 +268,8 @@ class Symbol(anytree.NodeMixin):
 
     @property
     def domain(self):
-        """list of applicable domains
+        """
+        list of applicable domains.
 
         Returns
         -------
@@ -174,8 +300,8 @@ class Symbol(anytree.NodeMixin):
 
     @property
     def auxiliary_domains(self):
-        "Returns domains that are not the primary domain"
-        return {k: v for k, v in self._domains.items() if k != "primary"}
+        """Returns auxiliary domains."""
+        return self._auxiliary_domains
 
     @auxiliary_domains.setter
     def auxiliary_domains(self, auxiliary_domains):
@@ -193,23 +319,30 @@ class Symbol(anytree.NodeMixin):
         if len(set(values)) != len(values):
             raise pybamm.DomainError("All auxiliary domains must be different")
 
+        self._auxiliary_domains = auxiliary_domains.copy()
         self._domains.update(auxiliary_domains)
 
     @property
     def secondary_domain(self):
-        "Helper function to get the secondary domain of a symbol"
+        """Helper function to get the secondary domain of a symbol."""
         return self.auxiliary_domains["secondary"]
 
     def copy_domains(self, symbol):
-        "Copy the domains from a given symbol, bypassing checks"
-        self._domains = symbol.domains
+        """Copy the domains from a given symbol, bypassing checks."""
+        self._domains = symbol.domains.copy()
+        self._auxiliary_domains = {
+            k: v for k, v in self._domains.items() if k != "primary"
+        }
+        self.set_id()
 
     def clear_domains(self):
-        "Clear domains, bypassing checks"
+        """Clear domains, bypassing checks."""
         self._domains = {"primary": []}
+        self._auxiliary_domains = {}
+        self.set_id()
 
     def get_children_auxiliary_domains(self, children):
-        "Combine auxiliary domains from children, at all levels"
+        """Combine auxiliary domains from children, at all levels."""
         aux_domains = {}
         for child in children:
             for level in child.auxiliary_domains.keys():
@@ -257,11 +390,11 @@ class Symbol(anytree.NodeMixin):
         Returning new copies of the children, with parents removed to avoid corrupting
         the expression tree internal data
         """
-        return tuple([child.new_copy() for child in self.children])
+        return self._orphans
 
     def render(self):  # pragma: no cover
-        """print out a visual representation of the tree (this node and its
-        children)
+        """
+        Print out a visual representation of the tree (this node and its children)
         """
         for pre, _, node in anytree.RenderTree(self):
             if isinstance(node, pybamm.Scalar) and node.name != str(node.value):
@@ -279,7 +412,6 @@ class Symbol(anytree.NodeMixin):
 
         filename : str
             filename to output, must end in ".png"
-
         """
 
         # check that filename ends in .png.
@@ -292,13 +424,14 @@ class Symbol(anytree.NodeMixin):
             DotExporter(
                 new_node, nodeattrfunc=lambda node: 'label="{}"'.format(node.label)
             ).to_picture(filename)
-        except FileNotFoundError:
+        except FileNotFoundError:  # pragma: no cover
             # raise error but only through logger so that test passes
             pybamm.logger.error("Please install graphviz>=2.42.2 to use dot exporter")
 
     def relabel_tree(self, symbol, counter):
-        """ Finds all children of a symbol and assigns them a new id so that they can be
-                visualised properly using the graphviz output
+        """
+        Finds all children of a symbol and assigns them a new id so that they can be
+        visualised properly using the graphviz output
         """
         name = symbol.name
         if name == "div":
@@ -315,8 +448,6 @@ class Symbol(anytree.NodeMixin):
             name = "&#43;"
         elif name == "**":
             name = "^"
-        elif name == "epsilon_s":
-            name = "&#603;"
 
         new_node = anytree.Node(str(counter), label=name)
         counter += 1
@@ -330,8 +461,8 @@ class Symbol(anytree.NodeMixin):
         return new_node, counter
 
     def pre_order(self):
-        """returns an iterable that steps through the tree in pre-order
-        fashion
+        """
+        returns an iterable that steps through the tree in pre-order fashion.
 
         Examples
         --------
@@ -344,12 +475,11 @@ class Symbol(anytree.NodeMixin):
         *
         a
         b
-
         """
         return anytree.PreOrderIter(self)
 
     def __str__(self):
-        """return a string representation of the node and its children"""
+        """return a string representation of the node and its children."""
         return self._name
 
     def __repr__(self):
@@ -366,106 +496,136 @@ class Symbol(anytree.NodeMixin):
         )
 
     def __add__(self, other):
-        """return an :class:`Addition` object"""
-        return pybamm.simplify_if_constant(
-            pybamm.Addition(self, other), keep_domains=True
-        )
+        """return an :class:`Addition` object."""
+        return pybamm.simplified_addition(self, other)
 
     def __radd__(self, other):
-        """return an :class:`Addition` object"""
-        return pybamm.simplify_if_constant(
-            pybamm.Addition(other, self), keep_domains=True
-        )
+        """return an :class:`Addition` object."""
+        return pybamm.simplified_addition(other, self)
 
     def __sub__(self, other):
-        """return a :class:`Subtraction` object"""
-        return pybamm.simplify_if_constant(
-            pybamm.Subtraction(self, other), keep_domains=True
-        )
+        """return a :class:`Subtraction` object."""
+        return pybamm.simplified_subtraction(self, other)
 
     def __rsub__(self, other):
-        """return a :class:`Subtraction` object"""
-        return pybamm.simplify_if_constant(
-            pybamm.Subtraction(other, self), keep_domains=True
-        )
+        """return a :class:`Subtraction` object."""
+        return pybamm.simplified_subtraction(other, self)
 
     def __mul__(self, other):
-        """return a :class:`Multiplication` object"""
-        return pybamm.simplify_if_constant(
-            pybamm.Multiplication(self, other), keep_domains=True
-        )
+        """return a :class:`Multiplication` object."""
+        return pybamm.simplified_multiplication(self, other)
 
     def __rmul__(self, other):
-        """return a :class:`Multiplication` object"""
-        return pybamm.simplify_if_constant(
-            pybamm.Multiplication(other, self), keep_domains=True
-        )
+        """return a :class:`Multiplication` object."""
+        return pybamm.simplified_multiplication(other, self)
 
     def __matmul__(self, other):
-        """return a :class:`MatrixMultiplication` object"""
-        return pybamm.simplify_if_constant(
-            pybamm.MatrixMultiplication(self, other), keep_domains=True
-        )
+        """return a :class:`MatrixMultiplication` object."""
+        return pybamm.simplified_matrix_multiplication(self, other)
 
     def __rmatmul__(self, other):
-        """return a :class:`MatrixMultiplication` object"""
-        return pybamm.simplify_if_constant(
-            pybamm.MatrixMultiplication(other, self), keep_domains=True
-        )
+        """return a :class:`MatrixMultiplication` object."""
+        return pybamm.simplified_matrix_multiplication(other, self)
 
     def __truediv__(self, other):
-        """return a :class:`Division` object"""
-        return pybamm.simplify_if_constant(
-            pybamm.Division(self, other), keep_domains=True
-        )
+        """return a :class:`Division` object."""
+        return pybamm.simplified_division(self, other)
 
     def __rtruediv__(self, other):
-        """return a :class:`Division` object"""
-        return pybamm.simplify_if_constant(
-            pybamm.Division(other, self), keep_domains=True
-        )
+        """return a :class:`Division` object."""
+        return pybamm.simplified_division(other, self)
 
     def __pow__(self, other):
-        """return a :class:`Power` object"""
-        return pybamm.simplify_if_constant(pybamm.Power(self, other), keep_domains=True)
+        """return a :class:`Power` object."""
+        return pybamm.simplified_power(self, other)
 
     def __rpow__(self, other):
-        """return a :class:`Power` object"""
-        return pybamm.simplify_if_constant(pybamm.Power(other, self), keep_domains=True)
+        """return a :class:`Power` object."""
+        return pybamm.simplified_power(other, self)
 
     def __lt__(self, other):
-        """return a :class:`NotEqualHeaviside` object"""
-        return pybamm.simplify_if_constant(
-            pybamm.NotEqualHeaviside(self, other), keep_domains=True
-        )
+        """return a :class:`NotEqualHeaviside` object, or a smooth approximation."""
+        k = pybamm.settings.heaviside_smoothing
+        # Return exact approximation if that is the setting or the outcome is a constant
+        # (i.e. no need for smoothing)
+        if k == "exact" or (is_constant(self) and is_constant(other)):
+            out = pybamm.NotEqualHeaviside(self, other)
+        else:
+            out = pybamm.sigmoid(self, other, k)
+        return pybamm.simplify_if_constant(out)
 
     def __le__(self, other):
-        """return a :class:`EqualHeaviside` object"""
-        return pybamm.simplify_if_constant(
-            pybamm.EqualHeaviside(self, other), keep_domains=True
-        )
+        """return a :class:`EqualHeaviside` object, or a smooth approximation."""
+        k = pybamm.settings.heaviside_smoothing
+        # Return exact approximation if that is the setting or the outcome is a constant
+        # (i.e. no need for smoothing)
+        if k == "exact" or (is_constant(self) and is_constant(other)):
+            out = pybamm.EqualHeaviside(self, other)
+        else:
+            out = pybamm.sigmoid(self, other, k)
+        return pybamm.simplify_if_constant(out)
 
     def __gt__(self, other):
-        """return a :class:`NotEqualHeaviside` object"""
-        return pybamm.simplify_if_constant(
-            pybamm.NotEqualHeaviside(other, self), keep_domains=True
-        )
+        """return a :class:`NotEqualHeaviside` object, or a smooth approximation."""
+        k = pybamm.settings.heaviside_smoothing
+        # Return exact approximation if that is the setting or the outcome is a constant
+        # (i.e. no need for smoothing)
+        if k == "exact" or (is_constant(self) and is_constant(other)):
+            out = pybamm.NotEqualHeaviside(other, self)
+        else:
+            out = pybamm.sigmoid(other, self, k)
+        return pybamm.simplify_if_constant(out)
 
     def __ge__(self, other):
-        """return a :class:`EqualHeaviside` object"""
-        return pybamm.simplify_if_constant(
-            pybamm.EqualHeaviside(other, self), keep_domains=True
-        )
+        """return a :class:`EqualHeaviside` object, or a smooth approximation."""
+        k = pybamm.settings.heaviside_smoothing
+        # Return exact approximation if that is the setting or the outcome is a constant
+        # (i.e. no need for smoothing)
+        if k == "exact" or (is_constant(self) and is_constant(other)):
+            out = pybamm.EqualHeaviside(other, self)
+        else:
+            out = pybamm.sigmoid(other, self, k)
+        return pybamm.simplify_if_constant(out)
 
     def __neg__(self):
-        """return a :class:`Negate` object"""
-        return pybamm.simplify_if_constant(pybamm.Negate(self), keep_domains=True)
+        """return a :class:`Negate` object."""
+        if isinstance(self, pybamm.Negate):
+            # Double negative is a positive
+            return self.orphans[0]
+        elif isinstance(self, pybamm.Broadcast):
+            # Move negation inside the broadcast
+            # Apply recursively
+            return self._unary_new_copy(-self.orphans[0])
+        elif isinstance(self, pybamm.Concatenation) and all(
+            child.is_constant() for child in self.children
+        ):
+            return pybamm.concatenation(*[-child for child in self.orphans])
+        else:
+            return pybamm.simplify_if_constant(pybamm.Negate(self))
 
     def __abs__(self):
-        """return an :class:`AbsoluteValue` object"""
-        return pybamm.simplify_if_constant(
-            pybamm.AbsoluteValue(self), keep_domains=True
-        )
+        """return an :class:`AbsoluteValue` object, or a smooth approximation."""
+        if isinstance(self, pybamm.AbsoluteValue):
+            # No need to apply abs a second time
+            return self
+        elif isinstance(self, pybamm.Broadcast):
+            # Move absolute value inside the broadcast
+            # Apply recursively
+            abs_self_not_broad = pybamm.simplify_if_constant(abs(self.orphans[0]))
+            return self._unary_new_copy(abs_self_not_broad)
+        else:
+            k = pybamm.settings.abs_smoothing
+            # Return exact approximation if that is the setting or the outcome is a
+            # constant (i.e. no need for smoothing)
+            if k == "exact" or is_constant(self):
+                out = pybamm.AbsoluteValue(self)
+            else:
+                out = pybamm.smooth_absolute_value(self, k)
+            return pybamm.simplify_if_constant(out)
+
+    def __mod__(self, other):
+        """return an :class:`Modulo` object."""
+        return pybamm.simplify_if_constant(pybamm.Modulo(self, other))
 
     def diff(self, variable):
         """
@@ -478,7 +638,6 @@ class Symbol(anytree.NodeMixin):
         ----------
         variable : :class:`pybamm.Symbol`
             The variable with respect to which to differentiate
-
         """
         if variable.id == self.id:
             return pybamm.Scalar(1)
@@ -492,7 +651,9 @@ class Symbol(anytree.NodeMixin):
             return pybamm.Scalar(0)
 
     def _diff(self, variable):
-        "Default behaviour for differentiation, overriden by Binary and Unary Operators"
+        """
+        Default behaviour for differentiation, overriden by Binary and Unary Operators
+        """
         raise NotImplementedError
 
     def jac(self, variable, known_jacs=None, clear_domain=True):
@@ -517,7 +678,8 @@ class Symbol(anytree.NodeMixin):
         raise NotImplementedError
 
     def _base_evaluate(self, t=None, y=None, y_dot=None, inputs=None):
-        """evaluate expression tree
+        """
+        evaluate expression tree.
 
         will raise a ``NotImplementedError`` if this member function has not
         been defined for the node. For example, :class:`Scalar` returns its
@@ -535,7 +697,6 @@ class Symbol(anytree.NodeMixin):
         y_dot : numpy.array, optional
             array with time derivatives of state values to evaluate when solving
             (default None)
-
         """
         raise NotImplementedError(
             """method self.evaluate() not implemented
@@ -579,10 +740,12 @@ class Symbol(anytree.NodeMixin):
             return self._base_evaluate(t, y, y_dot, inputs)
 
     def evaluate_for_shape(self):
-        """Evaluate expression tree to find its shape. For symbols that cannot be
-        evaluated directly (e.g. `Variable` or `Parameter`), a vector of the appropriate
-        shape is returned instead, using the symbol's domain.
-        See :meth:`pybamm.Symbol.evaluate()`
+        """
+        Evaluate expression tree to find its shape.
+
+        For symbols that cannot be evaluated directly (e.g. `Variable` or `Parameter`),
+        a vector of the appropriate shape is returned instead, using the symbol's
+        domain. See :meth:`pybamm.Symbol.evaluate()`
         """
         try:
             return self._saved_evaluate_for_shape
@@ -591,29 +754,20 @@ class Symbol(anytree.NodeMixin):
             return self._saved_evaluate_for_shape
 
     def _evaluate_for_shape(self):
-        "See :meth:`Symbol.evaluate_for_shape`"
+        """See :meth:`Symbol.evaluate_for_shape`"""
         return self.evaluate()
 
     def is_constant(self):
-        """returns true if evaluating the expression is not dependent on `t` or `y`
-        or `u`
+        """
+        returns true if evaluating the expression is not dependent on `t` or `y`
+        or `inputs`
 
         See Also
         --------
         evaluate : evaluate the expression
-
         """
-        # if any of the nodes are instances of any of these types, then the whole
-        # expression depends on either t or y or u
-        search_types = (
-            pybamm.Variable,
-            pybamm.StateVector,
-            pybamm.Time,
-            pybamm.InputParameter,
-        )
-
-        # do the search, return true if no relevent nodes are found
-        return not self.has_symbol_of_classes(search_types)
+        # Default behaviour is False
+        return False
 
     def evaluate_ignoring_errors(self, t=0):
         """
@@ -622,11 +776,9 @@ class Symbol(anytree.NodeMixin):
         is returned. If there is an InputParameter in the tree then a 1 is returned.
         Otherwise the result of the evaluation is given.
 
-
         See Also
         --------
         evaluate : evaluate the expression
-
         """
         try:
             result = self.evaluate(t=t, inputs="shape test")
@@ -661,21 +813,17 @@ class Symbol(anytree.NodeMixin):
         See Also
         --------
         evaluate : evaluate the expression
-
         """
-        result = self.evaluate_ignoring_errors()
+        return self.shape_for_testing == ()
 
-        if isinstance(result, numbers.Number) or (
-            isinstance(result, np.ndarray) and np.prod(result.shape) == 1
-        ):
-            return True
-        else:
-            return False
+    def evaluates_to_constant_number(self):
+        return self.evaluates_to_number() and self.is_constant()
 
     def evaluates_on_edges(self, dimension):
         """
         Returns True if a symbol evaluates on an edge, i.e. symbol contains a gradient
         operator, but not a divergence operator, and is not an IndefiniteIntegral.
+        Caches the solution for faster results.
 
         Parameters
         ----------
@@ -689,11 +837,20 @@ class Symbol(anytree.NodeMixin):
             Whether the symbol evaluates on edges (in the finite volume discretisation
             sense)
         """
+        try:
+            return self._saved_evaluates_on_edges[dimension]
+        except KeyError:
+            eval_on_edges = self._evaluates_on_edges(dimension)
+            self._saved_evaluates_on_edges[dimension] = eval_on_edges
+            return eval_on_edges
+
+    def _evaluates_on_edges(self, dimension):
         # Default behaviour: return False
         return False
 
     def has_symbol_of_classes(self, symbol_classes):
-        """Returns True if equation has a term of the class(es) `symbol_class`.
+        """
+        Returns True if equation has a term of the class(es) `symbol_class`.
 
         Parameters
         ----------
@@ -702,9 +859,9 @@ class Symbol(anytree.NodeMixin):
         """
         return any(isinstance(symbol, symbol_classes) for symbol in self.pre_order())
 
-    def simplify(self, simplified_symbols=None):
-        """ Simplify the expression tree. See :class:`pybamm.Simplification`. """
-        return pybamm.Simplification(simplified_symbols).simplify(self)
+    def simplify(self, simplified_symbols=None, clear_domains=True):
+        """`simplify()` has now been removed."""
+        raise pybamm.ModelError("simplify is deprecated as it now has no effect")
 
     def to_casadi(self, t=None, y=None, y_dot=None, inputs=None, casadi_symbols=None):
         """
@@ -713,7 +870,7 @@ class Symbol(anytree.NodeMixin):
         """
         return pybamm.CasadiConverter(casadi_symbols).convert(self, t, y, y_dot, inputs)
 
-    def new_copy(self):
+    def create_copy(self):
         """
         Make a new copy of a symbol, to avoid Tree corruption errors while bypassing
         copy.deepcopy(), which is slow.
@@ -725,48 +882,60 @@ class Symbol(anytree.NodeMixin):
             )
         )
 
+    def new_copy(self):
+        """
+        Returns `create_copy` with added attributes
+        """
+        obj = self.create_copy()
+        obj._print_name = self.print_name
+        return obj
+
     @property
     def size(self):
         """
         Size of an object, found by evaluating it with appropriate t and y
         """
-        return np.prod(self.shape)
+        try:
+            return self._saved_size
+        except AttributeError:
+            self._saved_size = np.prod(self.shape)
+            return self._saved_size
 
     @property
     def shape(self):
         """
         Shape of an object, found by evaluating it with appropriate t and y.
         """
-        # Default behaviour is to try to evaluate the object directly
-        # Try with some large y, to avoid having to unpack (slow)
         try:
-            y = np.nan * np.ones((1000, 1))
-            evaluated_self = self.evaluate(0, y, y, inputs="shape test")
-        # If that fails, fall back to calculating how big y should really be
-        except ValueError:
-            unpacker = pybamm.SymbolUnpacker(pybamm.StateVector)
-            state_vectors_in_node = unpacker.unpack_symbol(self).values()
-            if state_vectors_in_node == []:
-                y = None
-            else:
+            return self._saved_shape
+        except AttributeError:
+            # Default behaviour is to try to evaluate the object directly
+            # Try with some large y, to avoid having to unpack (slow)
+            try:
+                y = np.nan * np.ones((1000, 1))
+                evaluated_self = self.evaluate(0, y, y, inputs="shape test")
+            # If that fails, fall back to calculating how big y should really be
+            except ValueError:
+                unpacker = pybamm.SymbolUnpacker(pybamm.StateVector)
+                state_vectors_in_node = unpacker.unpack_symbol(self).values()
                 min_y_size = max(
-                    len(x._evaluation_array) for x in state_vectors_in_node
+                    max(len(x._evaluation_array) for x in state_vectors_in_node), 1
                 )
                 # Pick a y that won't cause RuntimeWarnings
                 y = np.nan * np.ones((min_y_size, 1))
-            evaluated_self = self.evaluate(0, y, y, inputs="shape test")
+                evaluated_self = self.evaluate(0, y, y, inputs="shape test")
 
-        # Return shape of evaluated object
-        if isinstance(evaluated_self, numbers.Number):
-            return ()
-        else:
-            return evaluated_self.shape
+            # Return shape of evaluated object
+            if isinstance(evaluated_self, numbers.Number):
+                self._saved_shape = ()
+            else:
+                self._saved_shape = evaluated_self.shape
+
+        return self._saved_shape
 
     @property
     def size_for_testing(self):
-        """
-        Size of an object, based on shape for testing
-        """
+        """Size of an object, based on shape for testing."""
         return np.prod(self.shape_for_testing)
 
     @property
@@ -782,9 +951,17 @@ class Symbol(anytree.NodeMixin):
         else:
             return evaluated_self.shape
 
+    @property
+    def ndim_for_testing(self):
+        """
+        Number of dimensions of an object,
+        found by evaluating it with appropriate t and y
+        """
+        return len(self.shape_for_testing)
+
     def test_shape(self):
         """
-        Check that the discretised self has a pybamm `shape`, i.e. can be evaluated
+        Check that the discretised self has a pybamm `shape`, i.e. can be evaluated.
 
         Raises
         ------
@@ -795,3 +972,17 @@ class Symbol(anytree.NodeMixin):
             self.shape_for_testing
         except ValueError as e:
             raise pybamm.ShapeError("Cannot find shape (original error: {})".format(e))
+
+    @property
+    def print_name(self):
+        return self._print_name
+
+    @print_name.setter
+    def print_name(self, name):
+        if name is None:
+            self._print_name = name
+        else:
+            self._print_name = prettify_print_name(name)
+
+    def to_equation(self):
+        return sympy.Symbol(str(self.name))
