@@ -1,28 +1,23 @@
 #
 # Base model class
 #
-import inspect
 import numbers
-import pybamm
 import warnings
+from collections import OrderedDict
+
+import casadi
+import numpy as np
+
+import pybamm
+from pybamm.expression_tree.operations.latexify import Latexify
 
 
-class ParamClass:
-    """Class for converting a module of parameters into a class. For pickling."""
-
-    def __init__(self, methods):
-        for k, v in methods.__dict__.items():
-            # don't save module attributes (e.g. pybamm, numpy)
-            if not (k.startswith("__") or inspect.ismodule(v)):
-                self.__dict__[k] = v
-
-
-class BaseModel(object):
-    """Base model class for other models to extend.
+class BaseModel:
+    """
+    Base model class for other models to extend.
 
     Attributes
     ----------
-
     name: str
         A string giving the name of the model
     options: dict
@@ -78,10 +73,6 @@ class BaseModel(object):
         solver set up
     use_jacobian : bool
         Whether to use the Jacobian when solving the model (default is True)
-    use_simplify : bool
-        Whether to simplify the expression tress representing the rhs and
-        algebraic equations, Jacobain (if using) and events, before solving the
-        model (default is True)
     convert_to_format : str
         Whether to convert the expression trees representing the rhs and
         algebraic equations, Jacobain (if using) and events into a different format:
@@ -93,19 +84,18 @@ class BaseModel(object):
         algorithm to calculate the Jacobian.
 
         Default is "casadi".
-
     """
 
     def __init__(self, name="Unnamed model"):
         self.name = name
-        self.options = {}
+        self._options = {}
 
         # Initialise empty model
         self._rhs = {}
         self._algebraic = {}
         self._initial_conditions = {}
         self._boundary_conditions = {}
-        self._variables = {}
+        self._variables = pybamm.FuzzyDict({})
         self._events = []
         self._concatenated_rhs = None
         self._concatenated_algebraic = None
@@ -117,14 +107,16 @@ class BaseModel(object):
         self.external_variables = []
         self._parameters = None
         self._input_parameters = None
+        self._parameter_info = None
+        self._variables_casadi = {}
 
-        # Default behaviour is to use the jacobian and simplify
+        # Default behaviour is to use the jacobian
         self.use_jacobian = True
-        self.use_simplify = True
         self.convert_to_format = "casadi"
 
         # Model is not initially discretised
         self.is_discretised = False
+        self.y_slices = None
 
         # Default timescale is 1 second
         self.timescale = pybamm.Scalar(1)
@@ -178,6 +170,20 @@ class BaseModel(object):
 
     @variables.setter
     def variables(self, variables):
+        for name, var in variables.items():
+            if (
+                isinstance(var, pybamm.Variable)
+                and var.name != name
+                # Exception if the variable is also there under its own name
+                and not (var.name in variables and variables[var.name].id == var.id)
+                # Exception for the key "Leading-order"
+                and "leading-order" not in var.name.lower()
+                and "leading-order" not in name.lower()
+            ):
+                raise ValueError(
+                    f"Variable with name '{var.name}' is in variables dictionary with "
+                    f"name '{name}'. Names must match."
+                )
         self._variables = pybamm.FuzzyDict(variables)
 
     def variable_names(self):
@@ -261,12 +267,7 @@ class BaseModel(object):
 
     @param.setter
     def param(self, values):
-        if values is None:
-            self._param = None
-        else:
-            # convert module into a class
-            # (StackOverflow: https://tinyurl.com/yk3euon3)
-            self._param = ParamClass(values)
+        self._param = values
 
     @property
     def options(self):
@@ -278,68 +279,123 @@ class BaseModel(object):
 
     @property
     def timescale(self):
-        "Timescale of model, to be used for non-dimensionalising time when solving"
+        """Timescale of model, to be used for non-dimensionalising time when solving"""
         return self._timescale
 
     @timescale.setter
     def timescale(self, value):
-        "Set the timescale"
+        """Set the timescale"""
         self._timescale = value
 
     @property
-    def parameters(self):
-        "Returns all the parameters in the model"
-        if self._parameters is None:
-            self._parameters = self._find_parameters()
-        return self._parameters
+    def length_scales(self):
+        "Length scales of model"
+        return self._length_scale
 
-    def _find_parameters(self):
-        "Find all the parameters in the model"
-        unpacker = pybamm.SymbolUnpacker((pybamm.Parameter, pybamm.InputParameter))
-        all_parameters = unpacker.unpack_list_of_symbols(
-            list(self.rhs.values())
-            + list(self.algebraic.values())
-            + list(self.initial_conditions.values())
-            + list(self.variables.values())
-            + [event.expression for event in self.events]
+    @length_scales.setter
+    def length_scales(self, values):
+        "Set the length scale, converting any numbers to pybamm.Scalar"
+        for domain, scale in values.items():
+            if isinstance(scale, numbers.Number):
+                values[domain] = pybamm.Scalar(scale)
+        self._length_scale = values
+
+    @property
+    def parameters(self):
+        """Returns all the parameters in the model"""
+        self._parameters = self._find_symbols(
+            (pybamm.Parameter, pybamm.InputParameter, pybamm.FunctionParameter)
         )
-        return list(all_parameters.values())
+        return self._parameters
 
     @property
     def input_parameters(self):
-        "Returns all the input parameters in the model"
+        """Returns all the input parameters in the model"""
         if self._input_parameters is None:
-            self._input_parameters = self._find_input_parameters()
+            self._input_parameters = self._find_symbols(pybamm.InputParameter)
         return self._input_parameters
 
-    def _find_input_parameters(self):
-        "Find all the input parameters in the model"
-        unpacker = pybamm.SymbolUnpacker(pybamm.InputParameter)
+    def print_parameter_info(self):
+        self._parameter_info = ""
+        parameters = self._find_symbols(pybamm.Parameter)
+        for param in parameters:
+            self._parameter_info += f"{param.name} (Parameter)\n"
+        input_parameters = self._find_symbols(pybamm.InputParameter)
+        for input_param in input_parameters:
+            if input_param.domain == []:
+                self._parameter_info += f"{input_param.name} (InputParameter)\n"
+            else:
+                self._parameter_info += (
+                    f"{input_param.name} (InputParameter in {input_param.domain})\n"
+                )
+        function_parameters = self._find_symbols(pybamm.FunctionParameter)
+        for func_param in function_parameters:
+            # don't double count function parameters
+            if func_param.name not in self._parameter_info:
+                input_names = "'" + "', '".join(func_param.input_names) + "'"
+                self._parameter_info += (
+                    f"{func_param.name} (FunctionParameter "
+                    f"with input(s) {input_names})\n"
+                )
+
+        print(self._parameter_info)
+
+    def _find_symbols(self, typ):
+        """Find all the instances of `typ` in the model"""
+        unpacker = pybamm.SymbolUnpacker(typ)
         all_input_parameters = unpacker.unpack_list_of_symbols(
             list(self.rhs.values())
             + list(self.algebraic.values())
             + list(self.initial_conditions.values())
+            + [
+                x[side][0]
+                for x in self.boundary_conditions.values()
+                for side in x.keys()
+            ]
             + list(self.variables.values())
             + [event.expression for event in self.events]
+            + [self.timescale]
+            + list(self.length_scales.values())
         )
         return list(all_input_parameters.values())
 
     def __getitem__(self, key):
         return self.rhs[key]
 
-    def new_copy(self, build=False):
+    @property
+    def default_quick_plot_variables(self):
+        return None
+
+    def new_empty_copy(self):
         """
-        Create an empty copy with identical options, or new options if specified.
-        The 'build' parameter is included for compatibility with subclasses, but unused.
+        Create an empty copy of the model with the same name and "parameters"
+        (convert_to_format, etc), but empty equations and variables.
+        This is usually then called by :class:`pybamm.ParameterValues`,
+        :class:`pybamm.Discretisation`, or :class:`pybamm.SymbolReplacer`.
         """
         new_model = self.__class__(name=self.name)
         new_model.use_jacobian = self.use_jacobian
-        new_model.use_simplify = self.use_simplify
         new_model.convert_to_format = self.convert_to_format
         new_model.timescale = self.timescale
         new_model.length_scales = self.length_scales
+
+        # Variables from discretisation
+        new_model.is_discretised = self.is_discretised
+        new_model.y_slices = self.y_slices
+        new_model.concatenated_rhs = self.concatenated_rhs
+        new_model.concatenated_algebraic = self.concatenated_algebraic
+        new_model.concatenated_initial_conditions = self.concatenated_initial_conditions
+
         return new_model
 
+    def new_copy(self):
+        """
+        Creates an identical copy of the model, using the functionality of
+        :class:`pybamm.SymbolReplacer` but without performing any replacements
+        """
+        replacer = pybamm.SymbolReplacer({})
+        return replacer.process_model(self, inplace=False)
+
     def update(self, *submodels):
         """
         Update model to add new physics from submodels
@@ -362,6 +418,112 @@ class BaseModel(object):
             self.variables.update(submodel.variables)  # keys are strings so no check
             self._events += submodel.events
 
+    def set_initial_conditions_from(self, solution, inplace=True):
+        """
+        Update initial conditions with the final states from a Solution object or from
+        a dictionary.
+        This assumes that, for each variable in self.initial_conditions, there is a
+        corresponding variable in the solution with the same name and size.
+
+        Parameters
+        ----------
+        solution : :class:`pybamm.Solution`, or dict
+            The solution to use to initialize the model
+        inplace : bool
+            Whether to modify the model inplace or create a new model
+        """
+        if inplace is True:
+            model = self
+        else:
+            model = self.new_empty_copy()
+            model.rhs = self.rhs.copy()
+            model.algebraic = self.algebraic.copy()
+            model.initial_conditions = self.initial_conditions.copy()
+            model.boundary_conditions = self.boundary_conditions.copy()
+            model.variables = self.variables.copy()
+            model.events = self.events.copy()
+
+        if isinstance(solution, pybamm.Solution):
+            solution = solution.last_state
+        for var in self.initial_conditions:
+            if isinstance(var, pybamm.Variable):
+                try:
+                    final_state = solution[var.name]
+                except KeyError as e:
+                    raise pybamm.ModelError(
+                        "To update a model from a solution, each variable in "
+                        "model.initial_conditions must appear in the solution with "
+                        "the same key as the variable name. In the solution provided, "
+                        f"{e.args[0]}"
+                    )
+                if isinstance(solution, pybamm.Solution):
+                    final_state = final_state.data
+                if final_state.ndim == 0:
+                    final_state_eval = np.array([final_state])
+                elif final_state.ndim == 1:
+                    final_state_eval = final_state[-1:]
+                elif final_state.ndim == 2:
+                    final_state_eval = final_state[:, -1]
+                elif final_state.ndim == 3:
+                    final_state_eval = final_state[:, :, -1].flatten(order="F")
+                else:
+                    raise NotImplementedError("Variable must be 0D, 1D, or 2D")
+                model.initial_conditions[var] = pybamm.Vector(final_state_eval)
+            elif isinstance(var, pybamm.Concatenation):
+                children = []
+                for child in var.orphans:
+                    try:
+                        final_state = solution[child.name]
+                    except KeyError as e:
+                        raise pybamm.ModelError(
+                            "To update a model from a solution, each variable in "
+                            "model.initial_conditions must appear in the solution with "
+                            "the same key as the variable name. In the solution "
+                            f"provided, {e.args[0]}"
+                        )
+                    if isinstance(solution, pybamm.Solution):
+                        final_state = final_state.data
+                    if final_state.ndim == 2:
+                        final_state_eval = final_state[:, -1]
+                    else:
+                        raise NotImplementedError(
+                            "Variable in concatenation must be 1D"
+                        )
+                    children.append(final_state_eval)
+                model.initial_conditions[var] = pybamm.Vector(np.concatenate(children))
+
+            else:
+                raise NotImplementedError(
+                    "Variable must have type 'Variable' or 'Concatenation'"
+                )
+
+        # Also update the concatenated initial conditions if the model is already
+        # discretised
+        if self.is_discretised:
+            # Unpack slices for sorting
+            y_slices = {var.id: slce for var, slce in self.y_slices.items()}
+            slices = []
+            for symbol in self.initial_conditions.keys():
+                if isinstance(symbol, pybamm.Concatenation):
+                    # must append the slice for the whole concatenation, so that
+                    # equations get sorted correctly
+                    slices.append(
+                        slice(
+                            y_slices[symbol.children[0].id][0].start,
+                            y_slices[symbol.children[-1].id][0].stop,
+                        )
+                    )
+                else:
+                    slices.append(y_slices[symbol.id][0])
+            equations = list(model.initial_conditions.values())
+            # sort equations according to slices
+            sorted_equations = [eq for _, eq in sorted(zip(slices, equations))]
+            model.concatenated_initial_conditions = pybamm.NumpyConcatenation(
+                *sorted_equations
+            )
+
+        return model
+
     def check_and_combine_dict(self, dict1, dict2):
         # check that the key ids are distinct
         ids1 = set(x.id for x in dict1.keys())
@@ -431,7 +593,7 @@ class BaseModel(object):
                     )
 
     def check_well_determined(self, post_discretisation):
-        """ Check that the model is not under- or over-determined. """
+        """Check that the model is not under- or over-determined."""
         # Equations (differential and algebraic)
         # Get all the variables from differential and algebraic equations
         vars_in_rhs_keys = set()
@@ -510,43 +672,23 @@ class BaseModel(object):
 
     def check_algebraic_equations(self, post_discretisation):
         """
-        Check that the algebraic equations are well-posed.
-        Before discretisation, each algebraic equation key must appear in the equation
-        After discretisation, there must be at least one StateVector in each algebraic
-        equation
+        Check that the algebraic equations are well-posed. After discretisation,
+        there must be at least one StateVector in each algebraic equation.
         """
-        vars_in_bcs = set()
-        unpacker = pybamm.SymbolUnpacker(pybamm.Variable)
-        for side_eqn in self.boundary_conditions.values():
-            all_vars = unpacker.unpack_list_of_symbols(
-                [eqn for eqn, _ in side_eqn.values()]
-            )
-            vars_in_bcs.update(all_vars.keys())
-        if not post_discretisation:
-            # After the model has been defined, each algebraic equation key should
-            # appear in that algebraic equation, or in the boundary conditions
-            # this has been relaxed for concatenations for now
-            for var, eqn in self.algebraic.items():
-                if not (
-                    any(x.id == var.id for x in eqn.pre_order())
-                    or var.id in vars_in_bcs
-                    or isinstance(var, pybamm.Concatenation)
-                ):
-                    raise pybamm.ModelError(
-                        "each variable in the algebraic eqn keys must appear in the eqn"
-                    )
-        else:
-            # variables in keys don't get discretised so they will no longer match
-            # with the state vectors in the algebraic equations. Instead, we check
-            # that each algebraic equation contains some StateVector
+        if post_discretisation:
+            # Check that each algebraic equation contains some StateVector
             for eqn in self.algebraic.values():
                 if not eqn.has_symbol_of_classes(pybamm.StateVector):
                     raise pybamm.ModelError(
                         "each algebraic equation must contain at least one StateVector"
                     )
+        else:
+            # We do not perfom any checks before discretisation (most problematic
+            # cases should be caught by `check_well_determined`)
+            pass
 
     def check_ics_bcs(self):
-        """ Check that the initial and boundary conditions are well-posed. """
+        """Check that the initial and boundary conditions are well-posed."""
         # Initial conditions
         for var in self.rhs.keys():
             if var not in self.initial_conditions.keys():
@@ -554,27 +696,8 @@ class BaseModel(object):
                     """no initial condition given for variable '{}'""".format(var)
                 )
 
-        # Boundary conditions
-        for var, eqn in {**self.rhs, **self.algebraic}.items():
-            if eqn.has_symbol_of_classes(
-                (pybamm.Gradient, pybamm.Divergence)
-            ) and not eqn.has_symbol_of_classes(pybamm.Integral):
-                # I have relaxed this check for now so that the lumped temperature
-                # equation doesn't raise errors (this has and average in it)
-
-                # Variable must be in the boundary conditions
-                if not any(
-                    var.id == x.id
-                    for symbol in self.boundary_conditions.keys()
-                    for x in symbol.pre_order()
-                ):
-                    raise pybamm.ModelError(
-                        "no boundary condition given for "
-                        "variable '{}' with equation '{}'.".format(var, eqn)
-                    )
-
     def check_default_variables_dictionaries(self):
-        """ Chec that the right variables are provided. """
+        """Check that the right variables are provided."""
         missing_vars = []
         for output, expression in self._variables.items():
             if expression is None:
@@ -624,7 +747,7 @@ class BaseModel(object):
                 )
 
     def check_no_repeated_keys(self):
-        "Check that no equation keys are repeated"
+        """Check that no equation keys are repeated."""
         rhs_alg = {**self.rhs, **self.algebraic}
         rhs_alg_keys = []
 
@@ -664,6 +787,160 @@ class BaseModel(object):
 
         print(div)
 
+    def check_discretised_or_discretise_inplace_if_0D(self):
+        """
+        Discretise model if it isn't already discretised
+        This only works with purely 0D models, as otherwise the mesh and spatial
+        method should be specified by the user
+        """
+        if self.is_discretised is False:
+            try:
+                disc = pybamm.Discretisation()
+                disc.process_model(self)
+            except pybamm.DiscretisationError as e:
+                raise pybamm.DiscretisationError(
+                    "Cannot automatically discretise model, model should be "
+                    "discretised before exporting casadi functions ({})".format(e)
+                )
+
+    def export_casadi_objects(self, variable_names, input_parameter_order=None):
+        """
+        Export the constituent parts of the model (rhs, algebraic, initial conditions,
+        etc) as casadi objects.
+
+        Parameters
+        ----------
+        variable_names : list
+            Variables to be exported alongside the model structure
+        input_parameter_order : list, optional
+            Order in which the input parameters should be stacked. If None, the order
+            returned by :meth:`BaseModel.input_parameters` is used
+
+        Returns
+        -------
+        casadi_dict : dict
+            Dictionary of {str: casadi object} pairs representing the model in casadi
+            format
+        """
+        self.check_discretised_or_discretise_inplace_if_0D()
+
+        # Create casadi functions for the model
+        t_casadi = casadi.MX.sym("t")
+        y_diff = casadi.MX.sym("y_diff", self.concatenated_rhs.size)
+        y_alg = casadi.MX.sym("y_alg", self.concatenated_algebraic.size)
+        y_casadi = casadi.vertcat(y_diff, y_alg)
+
+        # Read inputs
+        inputs_wrong_order = {}
+        for input_param in self.input_parameters:
+            name = input_param.name
+            inputs_wrong_order[name] = casadi.MX.sym(name, input_param._expected_size)
+        # Read external variables
+        external_casadi = {}
+        for external_varaiable in self.external_variables:
+            name = external_varaiable.name
+            ev_size = external_varaiable._evaluate_for_shape().shape[0]
+            external_casadi[name] = casadi.MX.sym(name, ev_size)
+        # Sort according to input_parameter_order
+        if input_parameter_order is None:
+            inputs = inputs_wrong_order
+        else:
+            inputs = {name: inputs_wrong_order[name] for name in input_parameter_order}
+        # Set up external variables and inputs
+        # Put external variables first like the integrator expects
+        ext_and_in = {**external_casadi, **inputs}
+        inputs_stacked = casadi.vertcat(*[p for p in ext_and_in.values()])
+
+        # Convert initial conditions to casadi form
+        y0 = self.concatenated_initial_conditions.to_casadi(
+            t_casadi, y_casadi, inputs=inputs
+        )
+        x0 = y0[: self.concatenated_rhs.size]
+        z0 = y0[self.concatenated_rhs.size :]
+
+        # Convert rhs and algebraic to casadi form and calculate jacobians
+        rhs = self.concatenated_rhs.to_casadi(t_casadi, y_casadi, inputs=ext_and_in)
+        jac_rhs = casadi.jacobian(rhs, y_casadi)
+        algebraic = self.concatenated_algebraic.to_casadi(
+            t_casadi, y_casadi, inputs=inputs
+        )
+        jac_algebraic = casadi.jacobian(algebraic, y_casadi)
+
+        # For specified variables, convert to casadi
+        variables = OrderedDict()
+        for name in variable_names:
+            var = self.variables[name]
+            variables[name] = var.to_casadi(t_casadi, y_casadi, inputs=ext_and_in)
+
+        casadi_dict = {
+            "t": t_casadi,
+            "x": y_diff,
+            "z": y_alg,
+            "inputs": inputs_stacked,
+            "rhs": rhs,
+            "algebraic": algebraic,
+            "jac_rhs": jac_rhs,
+            "jac_algebraic": jac_algebraic,
+            "variables": variables,
+            "x0": x0,
+            "z0": z0,
+        }
+
+        return casadi_dict
+
+    def generate(
+        self, filename, variable_names, input_parameter_order=None, cg_options=None
+    ):
+        """
+        Generate the model in C, using CasADi.
+
+        Parameters
+        ----------
+        filename : str
+            Name of the file to which to save the code
+        variable_names : list
+            Variables to be exported alongside the model structure
+        input_parameter_order : list, optional
+            Order in which the input parameters should be stacked. If None, the order
+            returned by :meth:`BaseModel.input_parameters` is used
+        cg_options : dict
+            Options to pass to the code generator.
+            See https://web.casadi.org/docs/#generating-c-code
+        """
+        model = self.export_casadi_objects(variable_names, input_parameter_order)
+
+        # Read the exported objects
+        t, x, z, p = model["t"], model["x"], model["z"], model["inputs"]
+        x0, z0 = model["x0"], model["z0"]
+        rhs, alg = model["rhs"], model["algebraic"]
+        variables = model["variables"]
+        jac_rhs, jac_alg = model["jac_rhs"], model["jac_algebraic"]
+
+        # Create functions
+        rhs_fn = casadi.Function("rhs_", [t, x, z, p], [rhs])
+        alg_fn = casadi.Function("alg_", [t, x, z, p], [alg])
+        jac_rhs_fn = casadi.Function("jac_rhs", [t, x, z, p], [jac_rhs])
+        jac_alg_fn = casadi.Function("jac_alg", [t, x, z, p], [jac_alg])
+        # Call these functions to initialize initial conditions
+        # (initial conditions are not yet consistent at this stage)
+        x0_fn = casadi.Function("x0", [p], [x0])
+        z0_fn = casadi.Function("z0", [p], [z0])
+        # Variables
+        variables_stacked = casadi.vertcat(*variables.values())
+        variables_fn = casadi.Function("variables", [t, x, z, p], [variables_stacked])
+
+        # Write C files
+        cg_options = cg_options or {}
+        C = casadi.CodeGenerator(filename, cg_options)
+        C.add(rhs_fn)
+        C.add(alg_fn)
+        C.add(jac_rhs_fn)
+        C.add(jac_alg_fn)
+        C.add(x0_fn)
+        C.add(z0_fn)
+        C.add(variables_fn)
+        C.generate()
+
     @property
     def default_parameter_values(self):
         return pybamm.ParameterValues({})
@@ -686,12 +963,19 @@ class BaseModel(object):
 
     @property
     def default_solver(self):
-        "Return default solver based on whether model is ODE model or DAE model"
-        if len(self.algebraic) == 0:
-            return pybamm.ScipySolver()
+        """Return default solver based on whether model is ODE/DAE or algebraic"""
+        if len(self.rhs) == 0 and len(self.algebraic) != 0:
+            return pybamm.CasadiAlgebraicSolver()
         else:
             return pybamm.CasadiSolver(mode="safe")
 
+    def latexify(self, filename=None, newline=True):
+        # For docstring, see pybamm.expression_tree.operations.latexify.Latexify
+        return Latexify(self, filename, newline).latexify()
+
+    # Set :meth:`latexify` docstring from :class:`Latexify`
+    latexify.__doc__ = Latexify.__doc__
+
 
 # helper functions for finding symbols
 def find_symbol_in_tree(tree, name):
@@ -726,7 +1010,7 @@ class EquationDict(dict):
         super().__init__(equations)
 
     def __setitem__(self, key, value):
-        "Call the update functionality when doing a setitem"
+        """Call the update functionality when doing a setitem."""
         self.update({key: value})
 
     def update(self, equations):
@@ -782,7 +1066,7 @@ class BoundaryConditionsDict(dict):
         super().__init__(bcs)
 
     def __setitem__(self, key, value):
-        "Call the update functionality when doing a setitem"
+        """Call the update functionality when doing a setitem."""
         self.update({key: value})
 
     def update(self, bcs):
@@ -790,7 +1074,7 @@ class BoundaryConditionsDict(dict):
         super().update(bcs)
 
     def check_and_convert_bcs(self, boundary_conditions):
-        """ Convert any scalar bcs in dict to 'pybamm.Scalar', and check types """
+        """Convert any scalar bcs in dict to 'pybamm.Scalar', and check types."""
         # Convert any numbers to a pybamm.Scalar
         for var, bcs in boundary_conditions.items():
             for side, bc in bcs.items():
