@@ -15,9 +15,13 @@ class BaseModel(pybamm.BaseBatteryModel):
 
     """
 
-    def __init__(self, options=None, name="Unnamed lead-acid model"):
+    def __init__(self, options=None, name="Unnamed lead-acid model", build=False):
+        options = options or {}
+        # Specify that there are no particles in lead-acid, and no half-cell models
+        options["particle shape"] = "no particles"
+        self.half_cell = False
         super().__init__(options, name)
-        self.param = pybamm.standard_parameters_lead_acid
+        self.param = pybamm.LeadAcidParameters()
 
         # Default timescale is discharge timescale
         self.timescale = self.param.tau_discharge
@@ -27,7 +31,7 @@ class BaseModel(pybamm.BaseBatteryModel):
             "negative electrode": self.param.L_x,
             "separator": self.param.L_x,
             "positive electrode": self.param.L_x,
-            "current collector y": self.param.L_y,
+            "current collector y": self.param.L_z,
             "current collector z": self.param.L_z,
         }
 
@@ -35,7 +39,7 @@ class BaseModel(pybamm.BaseBatteryModel):
 
     @property
     def default_parameter_values(self):
-        return pybamm.ParameterValues(chemistry=pybamm.parameter_sets.Sulzer2019)
+        return pybamm.ParameterValues("Sulzer2019")
 
     @property
     def default_geometry(self):
@@ -47,11 +51,21 @@ class BaseModel(pybamm.BaseBatteryModel):
     @property
     def default_var_pts(self):
         # Choose points that give uniform grid for the standard parameter values
-        var = pybamm.standard_spatial_vars
-        return {var.x_n: 25, var.x_s: 41, var.x_p: 34, var.y: 10, var.z: 10}
+        return {"x_n": 25, "x_s": 41, "x_p": 34, "y": 10, "z": 10}
+
+    @property
+    def default_quick_plot_variables(self):
+        return [
+            "Interfacial current density [A.m-2]",
+            "Electrolyte concentration [mol.m-3]",
+            "Current [A]",
+            "Porosity",
+            "Electrolyte potential [V]",
+            "Terminal voltage [V]",
+        ]
 
     def set_soc_variables(self):
-        "Set variables relating to the state of charge."
+        """Set variables relating to the state of charge."""
         # State of Charge defined as function of dimensionless electrolyte concentration
         z = pybamm.standard_spatial_vars.z
         soc = (
@@ -67,7 +81,18 @@ class BaseModel(pybamm.BaseBatteryModel):
             self.rhs[fci] = -self.variables["Total current density"] * 100
             self.initial_conditions[fci] = self.param.q_init * 100
 
+    def set_active_material_submodel(self):
+        self.submodels["negative active material"] = pybamm.active_material.Constant(
+            self.param, "Negative", self.options
+        )
+        self.submodels["positive active material"] = pybamm.active_material.Constant(
+            self.param, "Positive", self.options
+        )
+
     def set_sei_submodel(self):
 
-        self.submodels["negative sei"] = pybamm.sei.NoSEI(self.param, "Negative")
-        self.submodels["positive sei"] = pybamm.sei.NoSEI(self.param, "Positive")
+        self.submodels["sei"] = pybamm.sei.NoSEI(self.param)
+
+    def set_lithium_plating_submodel(self):
+
+        self.submodels["lithium plating"] = pybamm.lithium_plating.NoPlating(self.param)
