@@ -35,16 +35,18 @@ class LOQS(BaseModel):
 
         self.set_external_circuit_submodel()
         self.set_interfacial_submodel()
+        self.set_interface_utilisation_submodel()
         self.set_convection_submodel()
         self.set_porosity_submodel()
+        self.set_active_material_submodel()
         self.set_tortuosity_submodels()
-        self.set_negative_electrode_submodel()
         self.set_electrolyte_submodel()
-        self.set_positive_electrode_submodel()
+        self.set_electrode_submodels()
         self.set_thermal_submodel()
         self.set_side_reaction_submodels()
         self.set_current_collector_submodel()
         self.set_sei_submodel()
+        self.set_lithium_plating_submodel()
 
         if build:
             self.build_model()
@@ -52,7 +54,7 @@ class LOQS(BaseModel):
         if self.options["dimensionality"] == 0:
             self.use_jacobian = False
 
-        pybamm.citations.register("sulzer2019asymptotic")
+        pybamm.citations.register("Sulzer2019asymptotic")
 
     def set_external_circuit_submodel(self):
         """
@@ -94,8 +96,8 @@ class LOQS(BaseModel):
 
     def set_porosity_submodel(self):
 
-        self.submodels["leading-order porosity"] = pybamm.porosity.LeadingOrder(
-            self.param
+        self.submodels["leading-order porosity"] = pybamm.porosity.ReactionDrivenODE(
+            self.param, self.options, True
         )
 
     def set_tortuosity_submodels(self):
@@ -108,7 +110,7 @@ class LOQS(BaseModel):
 
     def set_convection_submodel(self):
 
-        if self.options["convection"] is False:
+        if self.options["convection"] == "none":
             self.submodels[
                 "leading-order transverse convection"
             ] = pybamm.convection.transverse.NoConvection(self.param)
@@ -130,16 +132,16 @@ class LOQS(BaseModel):
 
     def set_interfacial_submodel(self):
 
-        if self.options["surface form"] is False:
+        if self.options["surface form"] == "false":
             self.submodels[
                 "leading-order negative interface"
             ] = pybamm.interface.InverseButlerVolmer(
-                self.param, "Negative", "lead-acid main"
+                self.param, "Negative", "lead-acid main", self.options
             )
             self.submodels[
                 "leading-order positive interface"
             ] = pybamm.interface.InverseButlerVolmer(
-                self.param, "Positive", "lead-acid main"
+                self.param, "Positive", "lead-acid main", self.options
             )
             self.submodels[
                 "negative interface current"
@@ -154,68 +156,71 @@ class LOQS(BaseModel):
         else:
             self.submodels[
                 "leading-order negative interface"
-            ] = pybamm.interface.ButlerVolmer(self.param, "Negative", "lead-acid main")
+            ] = pybamm.interface.ButlerVolmer(
+                self.param, "Negative", "lead-acid main", self.options
+            )
 
             self.submodels[
                 "leading-order positive interface"
-            ] = pybamm.interface.ButlerVolmer(self.param, "Positive", "lead-acid main")
+            ] = pybamm.interface.ButlerVolmer(
+                self.param, "Positive", "lead-acid main", self.options
+            )
         # always use forward Butler-Volmer for the reaction submodel to be passed to the
         # higher order model
         self.reaction_submodels = {
             "Negative": [
-                pybamm.interface.ButlerVolmer(self.param, "Negative", "lead-acid main")
+                pybamm.interface.ButlerVolmer(
+                    self.param, "Negative", "lead-acid main", self.options
+                )
             ],
             "Positive": [
-                pybamm.interface.ButlerVolmer(self.param, "Positive", "lead-acid main")
+                pybamm.interface.ButlerVolmer(
+                    self.param, "Positive", "lead-acid main", self.options
+                )
             ],
         }
 
-    def set_negative_electrode_submodel(self):
+    def set_electrode_submodels(self):
 
         self.submodels[
-            "leading-order negative electrode"
+            "leading-order negative electrode potential"
         ] = pybamm.electrode.ohm.LeadingOrder(self.param, "Negative")
-
-    def set_positive_electrode_submodel(self):
-
         self.submodels[
-            "leading-order positive electrode"
+            "leading-order positive electrode potential"
         ] = pybamm.electrode.ohm.LeadingOrder(self.param, "Positive")
 
     def set_electrolyte_submodel(self):
 
         surf_form = pybamm.electrolyte_conductivity.surface_potential_form
 
-        if self.options["surface form"] is False:
+        if self.options["surface form"] == "false":
             self.submodels[
                 "leading-order electrolyte conductivity"
             ] = pybamm.electrolyte_conductivity.LeadingOrder(self.param)
-
+            surf_model = surf_form.Explicit
         elif self.options["surface form"] == "differential":
-            for domain in ["Negative", "Separator", "Positive"]:
-                self.submodels[
-                    "leading-order " + domain.lower() + " electrolyte conductivity"
-                ] = surf_form.LeadingOrderDifferential(self.param, domain)
-
+            surf_model = surf_form.LeadingOrderDifferential
         elif self.options["surface form"] == "algebraic":
-            for domain in ["Negative", "Separator", "Positive"]:
-                self.submodels[
-                    "leading-order " + domain.lower() + " electrolyte conductivity"
-                ] = surf_form.LeadingOrderAlgebraic(self.param, domain)
+            surf_model = surf_form.LeadingOrderAlgebraic
+
+        for domain in ["Negative", "Positive"]:
+            self.submodels[
+                domain.lower() + " surface potential difference"
+            ] = surf_model(self.param, domain)
 
         self.submodels[
             "electrolyte diffusion"
         ] = pybamm.electrolyte_diffusion.LeadingOrder(self.param)
 
     def set_side_reaction_submodels(self):
-        if "oxygen" in self.options["side reactions"]:
+        if self.options["hydrolysis"] == "true":
             self.submodels[
                 "leading-order oxygen diffusion"
             ] = pybamm.oxygen_diffusion.LeadingOrder(self.param)
             self.submodels[
                 "leading-order positive oxygen interface"
             ] = pybamm.interface.ForwardTafel(
-                self.param, "Positive", "lead-acid oxygen"
+                self.param, "Positive", "lead-acid oxygen", self.options
             )
             self.submodels[
                 "leading-order negative oxygen interface"
