@@ -28,7 +28,7 @@ class BasePotentialPair(BaseModel):
     def __init__(self, param):
         super().__init__(param)
 
-        pybamm.citations.register("timms2020")
+        pybamm.citations.register("Timms2021")
 
     def get_fundamental_variables(self):
 
@@ -81,7 +81,7 @@ class BasePotentialPair(BaseModel):
 
 
 class PotentialPair1plus1D(BasePotentialPair):
-    "Base class for a 1+1D potential pair model."
+    """Base class for a 1+1D potential pair model."""
 
     def __init__(self, param):
         super().__init__(param)
@@ -116,12 +116,12 @@ class PotentialPair1plus1D(BasePotentialPair):
         }
 
     def _get_effective_current_collector_area(self):
-        "In the 1+1D models the current collector effectively has surface area l_z"
+        """In the 1+1D models the current collector effectively has surface area l_z"""
         return self.param.l_z
 
 
 class PotentialPair2plus1D(BasePotentialPair):
-    "Base class for a 2+1D potential pair model"
+    """Base class for a 2+1D potential pair model"""
 
     def __init__(self, param):
         super().__init__(param)
@@ -180,5 +180,5 @@ class PotentialPair2plus1D(BasePotentialPair):
         }
 
     def _get_effective_current_collector_area(self):
-        "Return the area of the current collector"
+        """Return the area of the current collector."""
         return self.param.l_y * self.param.l_z
