@@ -16,14 +16,15 @@ class BaseModel(BaseElectrolyteConductivity):
         The parameters to use for this submodel
     domain : str
         The domain in which the model holds
-    reactions : dict
-        Dictionary of reaction terms
+    options : dict, optional
+        A dictionary of options to be passed to the model.
+
 
     **Extends:** :class:`pybamm.electrolyte_conductivity.BaseElectrolyteConductivity`
     """
 
-    def __init__(self, param, domain):
-        super().__init__(param, domain)
+    def __init__(self, param, domain, options=None):
+        super().__init__(param, domain, options)
 
     def get_fundamental_variables(self):
         if self.domain == "Negative":
@@ -33,7 +34,12 @@ class BaseModel(BaseElectrolyteConductivity):
         elif self.domain == "Positive":
             delta_phi = pybamm.standard_variables.delta_phi_p
 
-        variables = self._get_standard_surface_potential_difference_variables(delta_phi)
+        variables = self._get_standard_average_surface_potential_difference_variables(
+            pybamm.x_average(delta_phi)
+        )
+        variables.update(
+            self._get_standard_surface_potential_difference_variables(delta_phi)
+        )
 
         return variables
 
@@ -51,17 +57,15 @@ class BaseModel(BaseElectrolyteConductivity):
             T = variables[self.domain + " electrode temperature"]
 
             i_e = conductivity * (
-                ((1 + param.Theta * T) * param.chi(c_e) / c_e) * pybamm.grad(c_e)
+                ((1 + param.Theta * T) * param.chi(c_e, T) / c_e) * pybamm.grad(c_e)
                 + pybamm.grad(delta_phi)
                 + i_boundary_cc / sigma_eff
             )
-            variables.update(self._get_domain_current_variables(i_e))
+            variables[self.domain + " electrolyte current density"] = i_e
 
             phi_s = variables[self.domain + " electrode potential"]
             phi_e = phi_s - delta_phi
 
-            variables.update(self._get_domain_potential_variables(phi_e))
-
         elif self.domain == "Separator":
             x_s = pybamm.standard_spatial_vars.x_s
 
@@ -71,21 +75,17 @@ class BaseModel(BaseElectrolyteConductivity):
             tor_s = variables["Separator porosity"]
             T = variables["Separator temperature"]
 
-            chi_e_s = param.chi(c_e_s)
+            chi_e_s = param.chi(c_e_s, T)
             kappa_s_eff = param.kappa_e(c_e_s, T) * tor_s
 
-            phi_e_s = pybamm.boundary_value(
-                phi_e_n, "right"
-            ) + pybamm.IndefiniteIntegral(
+            phi_e = pybamm.boundary_value(phi_e_n, "right") + pybamm.IndefiniteIntegral(
                 (1 + param.Theta * T) * chi_e_s / c_e_s * pybamm.grad(c_e_s)
                 - param.C_e * i_boundary_cc / kappa_s_eff,
                 x_s,
             )
 
-            i_e_s = pybamm.PrimaryBroadcast(i_boundary_cc, "separator")
-
-            variables.update(self._get_domain_potential_variables(phi_e_s))
-            variables.update(self._get_domain_current_variables(i_e_s))
+            i_e = pybamm.PrimaryBroadcast(i_boundary_cc, "separator")
+            variables[self.domain + " electrolyte current density"] = i_e
 
             # Update boundary conditions (for indefinite integral)
             self.boundary_conditions[c_e_s] = {
@@ -93,8 +93,23 @@ class BaseModel(BaseElectrolyteConductivity):
                 "right": (pybamm.BoundaryGradient(c_e_s, "right"), "Neumann"),
             }
 
+        variables[self.domain + " electrolyte potential"] = phi_e
+
         if self.domain == "Positive":
-            variables.update(self._get_whole_cell_variables(variables))
+            phi_e_n = variables["Negative electrolyte potential"]
+            phi_e_s = variables["Separator electrolyte potential"]
+            phi_e_p = variables["Positive electrolyte potential"]
+
+            i_e_n = variables["Negative electrolyte current density"]
+            i_e_s = variables["Separator electrolyte current density"]
+            i_e_p = variables["Positive electrolyte current density"]
+            i_e = pybamm.concatenation(i_e_n, i_e_s, i_e_p)
+
+            variables.update(
+                self._get_standard_potential_variables(phi_e_n, phi_e_s, phi_e_p)
+            )
+            variables.update(self._get_standard_current_variables(i_e))
+            variables.update(self._get_electrolyte_overpotentials(variables))
 
         return variables
 
@@ -105,9 +120,9 @@ class BaseModel(BaseElectrolyteConductivity):
         c_e = variables[self.domain + " electrolyte concentration"]
         T = variables[self.domain + " electrode temperature"]
         if self.domain == "Negative":
-            sigma = param.sigma_n
+            sigma = param.sigma_n(T)
         elif self.domain == "Positive":
-            sigma = param.sigma_p
+            sigma = param.sigma_p(T)
 
         kappa_eff = param.kappa_e(c_e, T) * tor_e
         sigma_eff = sigma * tor_s
@@ -145,7 +160,7 @@ class BaseModel(BaseElectrolyteConductivity):
             flux_right = (
                 (i_boundary_cc / pybamm.BoundaryValue(conductivity, "right"))
                 - pybamm.BoundaryValue(
-                    (1 + param.Theta * T) * param.chi(c_e) / c_e, "right"
+                    (1 + param.Theta * T) * param.chi(c_e, T) / c_e, "right"
                 )
                 * c_e_flux
                 - i_boundary_cc * pybamm.BoundaryValue(1 / sigma_eff, "right")
@@ -162,7 +177,7 @@ class BaseModel(BaseElectrolyteConductivity):
             flux_left = (
                 (i_boundary_cc / pybamm.BoundaryValue(conductivity, "left"))
                 - pybamm.BoundaryValue(
-                    (1 + param.Theta * T) * param.chi(c_e) / c_e, "left"
+                    (1 + param.Theta * T) * param.chi(c_e, T) / c_e, "left"
                 )
                 * c_e_flux
                 - i_boundary_cc * pybamm.BoundaryValue(1 / sigma_eff, "left")
@@ -202,13 +217,14 @@ class FullAlgebraic(BaseModel):
     ----------
     param : parameter class
         The parameters to use for this submodel
+    options : dict, optional
+        A dictionary of options to be passed to the model.
 
-
-     **Extends:** :class:`pybamm.electrolyte_conductivity.surface_potential_form.BaseFull`
+    **Extends:** :class:`pybamm.electrolyte_conductivity.surface_potential_form.BaseFull`
     """  # noqa: E501
 
-    def __init__(self, param, domain):
-        super().__init__(param, domain)
+    def __init__(self, param, domain, options=None):
+        super().__init__(param, domain, options)
 
     def set_algebraic(self, variables):
         if self.domain == "Separator":
@@ -217,12 +233,16 @@ class FullAlgebraic(BaseModel):
         delta_phi = variables[self.domain + " electrode surface potential difference"]
         i_e = variables[self.domain + " electrolyte current density"]
 
+        # Get surface area to volume ratio (could be a distribution in x to
+        # account for graded electrodes)
+        a = variables[self.domain + " electrode surface area to volume ratio"]
+
         # Variable summing all of the interfacial current densities
         sum_j = variables[
             "Sum of " + self.domain.lower() + " electrode interfacial current densities"
         ]
 
-        self.algebraic[delta_phi] = pybamm.div(i_e) - sum_j
+        self.algebraic[delta_phi] = pybamm.div(i_e) - a * sum_j
 
 
 class FullDifferential(BaseModel):
@@ -234,14 +254,14 @@ class FullDifferential(BaseModel):
     ----------
     param : parameter class
         The parameters to use for this submodel
-
+    options : dict, optional
+        A dictionary of options to be passed to the model.
 
     **Extends:** :class:`pybamm.electrolyte_conductivity.surface_potential_form.BaseFull`
-
     """  # noqa: E501
 
-    def __init__(self, param, domain):
-        super().__init__(param, domain)
+    def __init__(self, param, domain, options=None):
+        super().__init__(param, domain, options)
 
     def set_rhs(self, variables):
         if self.domain == "Separator":
@@ -255,9 +275,13 @@ class FullDifferential(BaseModel):
         delta_phi = variables[self.domain + " electrode surface potential difference"]
         i_e = variables[self.domain + " electrolyte current density"]
 
+        # Get surface area to volume ratio (could be a distribution in x to
+        # account for graded electrodes)
+        a = variables[self.domain + " electrode surface area to volume ratio"]
+
         # Variable summing all of the interfacial current densities
         sum_j = variables[
             "Sum of " + self.domain.lower() + " electrode interfacial current densities"
         ]
 
-        self.rhs[delta_phi] = 1 / C_dl * (pybamm.div(i_e) - sum_j)
+        self.rhs[delta_phi] = 1 / C_dl * (pybamm.div(i_e) - a * sum_j)
