@@ -4,8 +4,8 @@ from .function_control_external_circuit import (
     FunctionControl,
     VoltageFunctionControl,
     PowerFunctionControl,
+    CCCVFunctionControl,
     LeadingOrderFunctionControl,
     LeadingOrderVoltageFunctionControl,
     LeadingOrderPowerFunctionControl,
 )
-from .experiment_events import ExperimentEvents
