@@ -37,14 +37,23 @@ class FirstOrderKinetics(BaseInterface):
         dj_ddeltaphi_0 = self.leading_order_model._get_dj_ddeltaphi(variables)
 
         # Update delta_phi with new phi_e and phi_s
-        variables = self._get_delta_phi(variables)
+        phi_s = variables[self.domain + " electrode potential"]
+        phi_e = variables[self.domain + " electrolyte potential"]
+        delta_phi = phi_s - phi_e
+        variables.update(
+            self._get_standard_average_surface_potential_difference_variables(
+                pybamm.x_average(delta_phi)
+            )
+        )
+        variables.update(
+            self._get_standard_surface_potential_difference_variables(delta_phi)
+        )
 
         delta_phi_0 = variables[
             "Leading-order "
             + self.domain.lower()
             + " electrode surface potential difference"
         ]
-        delta_phi = variables[self.domain + " electrode surface potential difference"]
         delta_phi_1 = (delta_phi - delta_phi_0) / self.param.C_e
 
         j_0 = variables[
