@@ -72,6 +72,11 @@ class InverseFirstOrderKinetics(BaseInterface):
         delta_phi = delta_phi_0 + self.param.C_e * delta_phi_1_av
 
         # Update variables dictionary
+        variables.update(
+            self._get_standard_average_surface_potential_difference_variables(
+                pybamm.x_average(delta_phi)
+            )
+        )
         variables.update(
             self._get_standard_surface_potential_difference_variables(delta_phi)
         )
