@@ -1,4 +1,4 @@
 from .base_kinetics import BaseKinetics
 from .butler_volmer import ButlerVolmer
-from .tafel import ForwardTafel, BackwardTafel
+from .tafel import ForwardTafel  # , BackwardTafel
 from .no_reaction import NoReaction
