@@ -19,27 +19,24 @@ class BaseKinetics(BaseInterface):
     reaction : str
         The name of the reaction being implemented
     options: dict
-        A dictionary of options to be passed to the model. In this case "sei film
-        resistance" is the important option. See :class:`pybamm.BaseBatteryModel`
+        A dictionary of options to be passed to the model.
+        See :class:`pybamm.BaseBatteryModel`
 
     **Extends:** :class:`pybamm.interface.BaseInterface`
     """
 
-    def __init__(self, param, domain, reaction, options=None):
-        super().__init__(param, domain, reaction)
-        if options is None:
-            options = {"sei film resistance": None}
-        self.options = options
+    def __init__(self, param, domain, reaction, options):
+        super().__init__(param, domain, reaction, options=options)
 
     def get_fundamental_variables(self):
         if (
-            self.options["sei film resistance"] == "distributed"
+            self.options["total interfacial current density as a state"] == "true"
             and "main" in self.reaction
         ):
             j = pybamm.Variable(
                 "Total "
                 + self.domain.lower()
-                + " electrode interfacial current density",
+                + " electrode interfacial current density variable",
                 domain=self.domain.lower() + " electrode",
                 auxiliary_domains={"secondary": "current collector"},
             )
@@ -54,13 +51,26 @@ class BaseKinetics(BaseInterface):
             return {}
 
     def get_coupled_variables(self, variables):
-        # Calculate delta_phi from phi_s and phi_e if it isn't already known
-        if self.domain + " electrode surface potential difference" not in variables:
-            variables = self._get_delta_phi(variables)
-        delta_phi = variables[self.domain + " electrode surface potential difference"]
-        # If delta_phi was broadcast, take only the orphan
-        if isinstance(delta_phi, pybamm.Broadcast):
-            delta_phi = delta_phi.orphans[0]
+        if self.reaction == "lithium metal plating":  # li metal electrode (half-cell)
+            delta_phi = variables[
+                "Lithium metal interface surface potential difference"
+            ]
+        else:
+            delta_phi = variables[
+                self.domain + " electrode surface potential difference"
+            ]
+            # If delta_phi was broadcast, take only the orphan.
+            if isinstance(delta_phi, pybamm.Broadcast):
+                delta_phi = delta_phi.orphans[0]
+        # For "particle-size distribution" models, delta_phi must then be
+        # broadcast to "particle size" domain
+        if (
+            self.reaction == "lithium-ion main"
+            and self.options["particle size"] == "distribution"
+        ):
+            delta_phi = pybamm.PrimaryBroadcast(
+                delta_phi, [self.domain.lower() + " particle size"]
+            )
 
         # Get exchange-current density
         j0 = self._get_exchange_current_density(variables)
@@ -72,46 +82,73 @@ class BaseKinetics(BaseInterface):
         j_tot_av = self._get_average_total_interfacial_current_density(variables)
         # j = j_tot_av + (j - pybamm.x_average(j))  # enforce true average
 
-        # Add SEI resistance
-        if self.options["sei film resistance"] == "distributed":
-            if self.domain == "Negative":
-                R_sei = self.param.R_sei_n
-            elif self.domain == "Positive":
-                R_sei = self.param.R_sei_p
-            L_sei = variables[
-                "Total " + self.domain.lower() + " electrode sei thickness"
-            ]
-            j_tot = variables[
-                "Total "
-                + self.domain.lower()
-                + " electrode interfacial current density variable"
-            ]
-            eta_sei = -j_tot * L_sei * R_sei
-        elif self.options["sei film resistance"] == "average":
-            if self.domain == "Negative":
-                R_sei = self.param.R_sei_n
-            elif self.domain == "Positive":
-                R_sei = self.param.R_sei_p
-            L_sei = variables[
-                "Total " + self.domain.lower() + " electrode sei thickness"
-            ]
-            eta_sei = -j_tot_av * L_sei * R_sei
-        else:
-            eta_sei = pybamm.Scalar(0)
-        eta_r += eta_sei
+        # Add SEI resistance in the negative electrode
+        if self.domain == "Negative":
+            if self.half_cell or self.options["SEI film resistance"] == "average":
+                R_sei = self.param.R_sei
+                L_sei = variables["Total SEI thickness"]
+                eta_sei = -j_tot_av * L_sei * R_sei
+            elif self.options["SEI film resistance"] == "distributed":
+                R_sei = self.param.R_sei
+                L_sei = variables["Total SEI thickness"]
+                j_tot = variables[
+                    "Total negative electrode interfacial current density variable"
+                ]
+                eta_sei = -j_tot * L_sei * R_sei
+            else:
+                eta_sei = pybamm.Scalar(0)
+            eta_r += eta_sei
 
         # Get number of electrons in reaction
         ne = self._get_number_of_electrons_in_reaction()
-        # Get kinetics. Note: T must have the same domain as j0 and eta_r
-        if j0.domain in ["current collector", ["current collector"]]:
+        # Get kinetics. Note: T and u must have the same domain as j0 and eta_r
+        if self.half_cell and self.domain == "Negative":
             T = variables["X-averaged cell temperature"]
+            u = variables["Lithium metal interface utilisation"]
+        elif j0.domain in ["current collector", ["current collector"]]:
+            T = variables["X-averaged cell temperature"]
+            u = variables[
+                "X-averaged " + self.domain.lower() + " electrode interface utilisation"
+            ]
+        elif j0.domain == [self.domain.lower() + " particle size"]:
+            if j0.domains["secondary"] != [self.domain.lower() + " electrode"]:
+                T = variables["X-averaged cell temperature"]
+                u = variables[
+                    "X-averaged "
+                    + self.domain.lower()
+                    + " electrode interface utilisation"
+                ]
+            else:
+                T = variables[self.domain + " electrode temperature"]
+                u = variables[self.domain + " electrode interface utilisation"]
+
+            # Broadcast T onto "particle size" domain
+            T = pybamm.PrimaryBroadcast(T, [self.domain.lower() + " particle size"])
         else:
             T = variables[self.domain + " electrode temperature"]
+            u = variables[self.domain + " electrode interface utilisation"]
 
         # Update j, except in the "distributed SEI resistance" model, where j will be
-        # found by solving an algebraic equation
+        # found by solving an algebraic equation.
         # (In the "distributed SEI resistance" model, we have already defined j)
-        j = self._get_kinetics(j0, ne, eta_r, T)
+        j = self._get_kinetics(j0, ne, eta_r, T, u)
+
+        if j.domain == [self.domain.lower() + " particle size"]:
+            # If j depends on particle size, get size-dependent "distribution"
+            # variables first
+            variables.update(
+                self._get_standard_size_distribution_interfacial_current_variables(j)
+            )
+            variables.update(
+                self._get_standard_size_distribution_exchange_current_variables(j0)
+            )
+            variables.update(
+                self._get_standard_size_distribution_overpotential_variables(eta_r)
+            )
+            variables.update(
+                self._get_standard_size_distribution_ocp_variables(ocp, dUdT)
+            )
+
         variables.update(self._get_standard_interfacial_current_variables(j))
 
         variables.update(
@@ -121,17 +158,30 @@ class BaseKinetics(BaseInterface):
         variables.update(self._get_standard_overpotential_variables(eta_r))
         variables.update(self._get_standard_ocp_variables(ocp, dUdT))
 
-        if "main" in self.reaction:
+        if self.domain == "Negative" and self.reaction in [
+            "lithium-ion main",
+            "lithium metal plating",
+            "lead-acid main",
+        ]:
             variables.update(
                 self._get_standard_sei_film_overpotential_variables(eta_sei)
             )
 
         if (
-            "Negative electrode" + self.reaction_name + " interfacial current density"
-            in variables
-            and "Positive electrode"
-            + self.reaction_name
-            + " interfacial current density"
+            (
+                self.half_cell
+                or (
+                    "Negative electrode"
+                    + self.reaction_name
+                    + " interfacial current density"
+                )
+                in variables
+            )
+            and (
+                "Positive electrode"
+                + self.reaction_name
+                + " interfacial current density"
+            )
             in variables
             and self.Reaction_icd not in variables
         ):
@@ -146,7 +196,7 @@ class BaseKinetics(BaseInterface):
 
     def set_algebraic(self, variables):
         if (
-            self.options["sei film resistance"] == "distributed"
+            self.options["total interfacial current density as a state"] == "true"
             and "main" in self.reaction
         ):
             j_tot_var = variables[
@@ -165,7 +215,7 @@ class BaseKinetics(BaseInterface):
 
     def set_initial_conditions(self, variables):
         if (
-            self.options["sei film resistance"] == "distributed"
+            self.options["total interfacial current density as a state"] == "true"
             and "main" in self.reaction
         ):
             param = self.param
@@ -174,10 +224,15 @@ class BaseKinetics(BaseInterface):
                 + self.domain.lower()
                 + " electrode interfacial current density variable"
             ]
+            current_at_0 = (
+                pybamm.FunctionParameter("Current function [A]", {"Time [s]": 0})
+                / param.I_typ
+                * pybamm.sign(param.I_typ)
+            )
             if self.domain == "Negative":
-                j_tot_av_init = param.current_with_time / param.l_n
+                j_tot_av_init = current_at_0 / param.l_n
             elif self.domain == "Positive":
-                j_tot_av_init = -param.current_with_time / param.l_p
+                j_tot_av_init = -current_at_0 / param.l_p
 
             self.initial_conditions[j_tot_var] = j_tot_av_init
 
@@ -186,10 +241,16 @@ class BaseKinetics(BaseInterface):
         Default to calculate derivative of interfacial current density with respect to
         concentration. Can be overwritten by specific kinetic functions.
         """
-        c_e, delta_phi, j0, ne, ocp, T = self._get_interface_variables_for_first_order(
-            variables
-        )
-        j = self._get_kinetics(j0, ne, delta_phi - ocp, T)
+        (
+            c_e,
+            delta_phi,
+            j0,
+            ne,
+            ocp,
+            T,
+            u,
+        ) = self._get_interface_variables_for_first_order(variables)
+        j = self._get_kinetics(j0, ne, delta_phi - ocp, T, u)
         return j.diff(c_e)
 
     def _get_dj_ddeltaphi(self, variables):
@@ -197,17 +258,20 @@ class BaseKinetics(BaseInterface):
         Default to calculate derivative of interfacial current density with respect to
         surface potential difference. Can be overwritten by specific kinetic functions.
         """
-        _, delta_phi, j0, ne, ocp, T = self._get_interface_variables_for_first_order(
+        _, delta_phi, j0, ne, ocp, T, u = self._get_interface_variables_for_first_order(
             variables
         )
-        j = self._get_kinetics(j0, ne, delta_phi - ocp, T)
+        j = self._get_kinetics(j0, ne, delta_phi - ocp, T, u)
         return j.diff(delta_phi)
 
     def _get_interface_variables_for_first_order(self, variables):
-        # This is a bit of a hack, but we need to multiply electrolyte concentration by
-        # one to differentiate it from the electrolyte concentration inside the
+        # This is a bit of a hack, but we need to wrap electrolyte concentration with
+        # the NotConstant class
+        # to differentiate it from the electrolyte concentration inside the
         # surface potential difference when taking j.diff(c_e) later on
-        c_e_0 = variables["Leading-order x-averaged electrolyte concentration"] * 1
+        c_e_0 = pybamm.NotConstant(
+            variables["Leading-order x-averaged electrolyte concentration"]
+        )
         hacked_variables = {
             **variables,
             self.domain
@@ -225,9 +289,13 @@ class BaseKinetics(BaseInterface):
         ocp = self._get_open_circuit_potential(hacked_variables)[0]
         if j0.domain in ["current collector", ["current collector"]]:
             T = variables["X-averaged cell temperature"]
+            u = variables[
+                "X-averaged " + self.domain.lower() + " electrode interface utilisation"
+            ]
         else:
             T = variables[self.domain + " electrode temperature"]
-        return c_e_0, delta_phi, j0, ne, ocp, T
+            u = variables[self.domain + " electrode interface utilisation"]
+        return c_e_0, delta_phi, j0, ne, ocp, T, u
 
     def _get_j_diffusion_limited_first_order(self, variables):
         """
