@@ -22,35 +22,41 @@ class ButlerVolmer(BaseKinetics):
     reaction : str
         The name of the reaction being implemented
     options: dict
-        A dictionary of options to be passed to the model. In this case "sei film
-        resistance" is the important option. See :class:`pybamm.BaseBatteryModel`
+        A dictionary of options to be passed to the model.
+        See :class:`pybamm.BaseBatteryModel`
 
     **Extends:** :class:`pybamm.interface.kinetics.BaseKinetics`
     """
 
-    def __init__(self, param, domain, reaction, options=None):
+    def __init__(self, param, domain, reaction, options):
         super().__init__(param, domain, reaction, options)
 
-    def _get_kinetics(self, j0, ne, eta_r, T):
+    def _get_kinetics(self, j0, ne, eta_r, T, u):
         prefactor = ne / (2 * (1 + self.param.Theta * T))
-        return 2 * j0 * pybamm.sinh(prefactor * eta_r)
+        return 2 * u * j0 * pybamm.sinh(prefactor * eta_r)
 
     def _get_dj_dc(self, variables):
-        "See :meth:`pybamm.interface.kinetics.BaseKinetics._get_dj_dc`"
-        c_e, delta_phi, j0, ne, ocp, T = self._get_interface_variables_for_first_order(
-            variables
-        )
+        """See :meth:`pybamm.interface.kinetics.BaseKinetics._get_dj_dc`"""
+        (
+            c_e,
+            delta_phi,
+            j0,
+            ne,
+            ocp,
+            T,
+            u,
+        ) = self._get_interface_variables_for_first_order(variables)
         eta_r = delta_phi - ocp
         prefactor = ne / (2 * (1 + self.param.Theta * T))
-        return (2 * j0.diff(c_e) * pybamm.sinh(prefactor * eta_r)) - (
-            2 * j0 * prefactor * ocp.diff(c_e) * pybamm.cosh(prefactor * eta_r)
+        return (2 * u * j0.diff(c_e) * pybamm.sinh(prefactor * eta_r)) - (
+            2 * u * j0 * prefactor * ocp.diff(c_e) * pybamm.cosh(prefactor * eta_r)
         )
 
     def _get_dj_ddeltaphi(self, variables):
-        "See :meth:`pybamm.interface.kinetics.BaseKinetics._get_dj_ddeltaphi`"
-        _, delta_phi, j0, ne, ocp, T = self._get_interface_variables_for_first_order(
+        """See :meth:`pybamm.interface.kinetics.BaseKinetics._get_dj_ddeltaphi`"""
+        _, delta_phi, j0, ne, ocp, T, u = self._get_interface_variables_for_first_order(
             variables
         )
         eta_r = delta_phi - ocp
         prefactor = ne / (2 * (1 + self.param.Theta * T))
-        return 2 * j0 * prefactor * pybamm.cosh(prefactor * eta_r)
+        return 2 * u * j0 * prefactor * pybamm.cosh(prefactor * eta_r)
