@@ -171,18 +171,12 @@ class FiniteVolume(pybamm.SpatialMethod):
         # check for particle domain
         if submesh.coord_sys == "spherical polar":
             second_dim_repeats = self._get_auxiliary_domain_repeats(symbol.domains)
-            edges = submesh.edges
 
-            # create np.array of repeated submesh.nodes
-            r_numpy = np.kron(np.ones(second_dim_repeats), submesh.nodes)
-            r_edges_numpy = np.kron(np.ones(second_dim_repeats), edges)
-
-            r = pybamm.Vector(r_numpy)
+            # create np.array of repeated submesh.edges
+            r_edges_numpy = np.kron(np.ones(second_dim_repeats), submesh.edges)
             r_edges = pybamm.Vector(r_edges_numpy)
 
-            out = (1 / (r ** 2)) * (
-                divergence_matrix @ ((r_edges ** 2) * discretised_symbol)
-            )
+            out = divergence_matrix @ ((r_edges ** 2) * discretised_symbol)
         else:
             out = divergence_matrix @ discretised_symbol
 
@@ -205,7 +199,14 @@ class FiniteVolume(pybamm.SpatialMethod):
         """
         # Create appropriate submesh by combining submeshes in domain
         submesh = self.mesh.combine_submeshes(*domains["primary"])
-        e = 1 / submesh.d_edges
+        if submesh.coord_sys == "spherical polar":
+            r_edges_left = submesh.edges[:-1]
+            r_edges_right = submesh.edges[1:]
+            d_edges = (r_edges_right ** 3 - r_edges_left ** 3) / 3
+        else:
+            d_edges = submesh.d_edges
+
+        e = 1 / d_edges
 
         # Create matrix using submesh
         n = submesh.npts + 1
@@ -230,21 +231,11 @@ class FiniteVolume(pybamm.SpatialMethod):
         return self.divergence(grad, grad, boundary_conditions)
 
     def integral(self, child, discretised_child, integration_dimension):
-        """Vector-vector dot product to implement the integral operator. """
+        """Vector-vector dot product to implement the integral operator."""
         integration_vector = self.definite_integral_matrix(
             child, integration_dimension=integration_dimension
         )
-
-        # Check for spherical domains
-        domain = child.domains[integration_dimension]
-        submesh = self.mesh.combine_submeshes(*domain)
-        if submesh.coord_sys == "spherical polar":
-            second_dim_repeats = self._get_auxiliary_domain_repeats(child.domains)
-            r_numpy = np.kron(np.ones(second_dim_repeats), submesh.nodes)
-            r = pybamm.Vector(r_numpy)
-            out = 4 * np.pi ** 2 * integration_vector @ (discretised_child * r)
-        else:
-            out = integration_vector @ discretised_child
+        out = integration_vector @ discretised_child
 
         return out
 
@@ -277,33 +268,40 @@ class FiniteVolume(pybamm.SpatialMethod):
             The finite volume integral matrix for the domain
         """
         domains = child.domains
+        if vector_type != "row" and integration_dimension == "secondary":
+            raise NotImplementedError(
+                "Integral in secondary vector only implemented in 'row' form"
+            )
+
+        domain = child.domains[integration_dimension]
+        submesh = self.mesh.combine_submeshes(*domain)
+        if submesh.coord_sys == "spherical polar":
+            r_edges_left = submesh.edges[:-1]
+            r_edges_right = submesh.edges[1:]
+            d_edges = 4 * np.pi * (r_edges_right ** 3 - r_edges_left ** 3) / 3
+        else:
+            d_edges = submesh.d_edges
+
         if integration_dimension == "primary":
             # Create appropriate submesh by combining submeshes in domain
             submesh = self.mesh.combine_submeshes(*domains["primary"])
 
             # Create vector of ones for primary domain submesh
-            vector = submesh.d_edges
 
             if vector_type == "row":
-                vector = vector[np.newaxis, :]
+                d_edges = d_edges[np.newaxis, :]
             elif vector_type == "column":
-                vector = vector[:, np.newaxis]
+                d_edges = d_edges[:, np.newaxis]
 
             # repeat matrix for each node in secondary dimensions
             second_dim_repeats = self._get_auxiliary_domain_repeats(domains)
             # generate full matrix from the submatrix
-            matrix = kron(eye(second_dim_repeats), vector)
+            matrix = kron(eye(second_dim_repeats), d_edges)
         elif integration_dimension == "secondary":
-            if vector_type != "row":
-                raise NotImplementedError(
-                    "Integral in secondary vector only implemented in 'row' form"
-                )
             # Create appropriate submesh by combining submeshes in domain
             primary_submesh = self.mesh.combine_submeshes(*domains["primary"])
-            secondary_submesh = self.mesh.combine_submeshes(*domains["secondary"])
 
             # Create matrix which integrates in the secondary dimension
-            d_edges = secondary_submesh.d_edges
             # Different number of edges depending on whether child evaluates on edges
             # in the primary dimensions
             if child.evaluates_on_edges("primary"):
@@ -312,9 +310,13 @@ class FiniteVolume(pybamm.SpatialMethod):
                 n_primary_pts = primary_submesh.npts
             int_matrix = hstack([d_edge * eye(n_primary_pts) for d_edge in d_edges])
 
-            # repeat matrix for each node in secondary dimensions
+            # repeat matrix for each node in higher dimensions
             third_dim_repeats = self._get_auxiliary_domain_repeats(
-                domains, tertiary_only=True
+                {
+                    k: v
+                    for k, v in domains.items()
+                    if (k == "tertiary" or k == "quaternary")
+                }
             )
             # generate full matrix from the submatrix
             matrix = kron(eye(third_dim_repeats), int_matrix)
@@ -326,7 +328,7 @@ class FiniteVolume(pybamm.SpatialMethod):
         return pybamm.Matrix(csr_matrix(matrix))
 
     def indefinite_integral(self, child, discretised_child, direction):
-        """Implementation of the indefinite integral operator. """
+        """Implementation of the indefinite integral operator."""
 
         # Different integral matrix depending on whether the integrand evaluates on
         # edges or nodes
@@ -405,7 +407,7 @@ class FiniteVolume(pybamm.SpatialMethod):
         **Backward integral**
 
         .. math::
-            F(x) = \\int_x^end\\!f(u)\\,du
+            F(x) = \\int_x^{end}\\!f(u)\\,du
 
         The indefinite integral must satisfy the following conditions:
 
@@ -815,13 +817,12 @@ class FiniteVolume(pybamm.SpatialMethod):
         # Need to match the domain. E.g. in the case of the boundary condition
         # on the particle, the gradient has domain particle but the bcs_vector
         # has domain electrode, since it is a function of the macroscopic variables
-        bcs_vector.domain = discretised_gradient.domain
-        bcs_vector.auxiliary_domains = discretised_gradient.auxiliary_domains
+        bcs_vector.copy_domains(discretised_gradient)
 
         # Make matrix which makes "gaps" in the the discretised gradient into
         # which the known Neumann values will be added. E.g. in 1D if the left
         # boundary condition is Dirichlet and the right Neumann, this matrix will
-        # act to append a zero to the end of the discretsied gradient
+        # act to append a zero to the end of the discretised gradient
         if lbc_type == "Neumann":
             left_vector = csr_matrix((1, n))
         else:
@@ -1130,8 +1131,9 @@ class FiniteVolume(pybamm.SpatialMethod):
             disc_left = self.node_to_edge(disc_left, method=method)
         # Return new binary operator with appropriate class
         out = pybamm.simplify_if_constant(
-            bin_op.__class__(disc_left, disc_right), keep_domains=True
+            bin_op._binary_new_copy(disc_left, disc_right)
         )
+
         return out
 
     def concatenation(self, disc_children):
@@ -1157,7 +1159,7 @@ class FiniteVolume(pybamm.SpatialMethod):
                         or n_edges (number of edges in the mesh)
                         """
                     )
-        return pybamm.DomainConcatenation(disc_children, self.mesh)
+        return pybamm.domain_concatenation(disc_children, self.mesh)
 
     def edge_to_node(self, discretised_symbol, method="arithmetic"):
         """
@@ -1375,7 +1377,7 @@ class FiniteVolume(pybamm.SpatialMethod):
                 raise ValueError("shift key '{}' not recognised".format(shift_key))
 
         # If discretised_symbol evaluates to number there is no need to average
-        if discretised_symbol.evaluates_to_number():
+        if discretised_symbol.size == 1:
             out = discretised_symbol
         elif method == "arithmetic":
             out = arithmetic_mean(discretised_symbol)
@@ -1384,3 +1386,66 @@ class FiniteVolume(pybamm.SpatialMethod):
         else:
             raise ValueError("method '{}' not recognised".format(method))
         return out
+
+    def upwind_or_downwind(self, symbol, discretised_symbol, bcs, direction):
+        """
+        Implement an upwinding operator. Currently, this requires the symbol to have
+        a Dirichlet boundary condition on the left side (for upwinding) or right side
+        (for downwinding).
+
+        Parameters
+        ----------
+        symbol : :class:`pybamm.SpatialVariable`
+            The variable to be discretised
+        discretised_gradient : :class:`pybamm.Vector`
+            Contains the discretised gradient of symbol
+        bcs : dict of tuples (:class:`pybamm.Scalar`, str)
+            Dictionary (with keys "left" and "right") of boundary conditions. Each
+            boundary condition consists of a value and a flag indicating its type
+            (e.g. "Dirichlet")
+        direction : str
+            Direction in which to apply the operator (upwind or downwind)
+        """
+        submesh = self.mesh.combine_submeshes(*symbol.domain)
+        n = submesh.npts
+
+        if symbol.id not in bcs:
+            raise pybamm.ModelError(
+                "Boundary conditions must be provided for "
+                "{}ing '{}'".format(direction, symbol)
+            )
+
+        if direction == "upwind":
+            bc, typ = bcs[symbol.id]["left"]
+            if typ != "Dirichlet":
+                raise pybamm.ModelError(
+                    "Dirichlet boundary conditions must be provided for "
+                    "upwinding '{}'".format(symbol)
+                )
+
+            concat_bc = pybamm.NumpyConcatenation(bc, discretised_symbol)
+
+            upwind_mat = vstack(
+                [
+                    csr_matrix(([1], ([0], [0])), shape=(1, n + 1)),
+                    diags([-0.5, 1.5], [0, 1], shape=(n, n + 1)),
+                ]
+            )
+            symbol_out = pybamm.Matrix(upwind_mat) @ concat_bc
+        elif direction == "downwind":
+            bc, typ = bcs[symbol.id]["right"]
+            if typ != "Dirichlet":
+                raise pybamm.ModelError(
+                    "Dirichlet boundary conditions must be provided for "
+                    "downwinding '{}'".format(symbol)
+                )
+
+            concat_bc = pybamm.NumpyConcatenation(discretised_symbol, bc)
+            downwind_mat = vstack(
+                [
+                    diags([1.5, -0.5], [0, 1], shape=(n, n + 1)),
+                    csr_matrix(([1], ([0], [n])), shape=(1, n + 1)),
+                ]
+            )
+            symbol_out = pybamm.Matrix(downwind_mat) @ concat_bc
+        return symbol_out
