@@ -40,11 +40,11 @@ class SpatialMethod:
             mesh[dom].npts_for_broadcast_to_nodes = mesh[dom].npts
         self._mesh = mesh
 
-    def _get_auxiliary_domain_repeats(self, auxiliary_domains, tertiary_only=False):
+    def _get_auxiliary_domain_repeats(self, auxiliary_domains):
         """
         Helper method to read the 'auxiliary_domain' meshes
         """
-        if tertiary_only is False and "secondary" in auxiliary_domains:
+        if "secondary" in auxiliary_domains:
             sec_mesh_npts = self.mesh.combine_submeshes(
                 *auxiliary_domains["secondary"]
             ).npts
@@ -56,7 +56,13 @@ class SpatialMethod:
             ).npts
         else:
             tert_mesh_npts = 1
-        return sec_mesh_npts * tert_mesh_npts
+        if "quaternary" in auxiliary_domains:
+            quat_mesh_npts = self.mesh.combine_submeshes(
+                *auxiliary_domains["quaternary"]
+            ).npts
+        else:
+            quat_mesh_npts = 1
+        return sec_mesh_npts * tert_mesh_npts * quat_mesh_npts
 
     @property
     def mesh(self):
@@ -102,7 +108,8 @@ class SpatialMethod:
             The auxiliary domains for broadcasting
         broadcast_type : str
             The type of broadcast: 'primary to node', 'primary to edges', 'secondary to
-            nodes', 'secondary to edges', 'full to nodes' or 'full to edges'
+            nodes', 'secondary to edges', 'tertiary to nodes', 'tertiary to edges',
+            'full to nodes' or 'full to edges'
 
         Returns
         -------
@@ -113,13 +120,20 @@ class SpatialMethod:
         primary_domain_size = sum(
             self.mesh[dom].npts_for_broadcast_to_nodes for dom in domain
         )
-        secondary_domain_size = self._get_auxiliary_domain_repeats(auxiliary_domains)
-        full_domain_size = primary_domain_size * secondary_domain_size
+        secondary_domain_size = self._get_auxiliary_domain_repeats(
+            {k: v for k, v in auxiliary_domains.items() if k == "secondary"}
+        )
+        tertiary_domain_size = self._get_auxiliary_domain_repeats(
+            {k: v for k, v in auxiliary_domains.items() if k == "tertiary"}
+        )
+        auxiliary_domains_size = self._get_auxiliary_domain_repeats(auxiliary_domains)
+        full_domain_size = primary_domain_size * auxiliary_domains_size
         if broadcast_type.endswith("to edges"):
             # add one point to each domain for broadcasting to edges
             primary_domain_size += 1
-            full_domain_size = primary_domain_size * secondary_domain_size
+            full_domain_size = primary_domain_size * auxiliary_domains_size
             secondary_domain_size += 1
+            tertiary_domain_size += 1
 
         if broadcast_type.startswith("primary"):
             # Make copies of the child stacked on top of each other
@@ -136,10 +150,15 @@ class SpatialMethod:
             identity = eye(symbol.shape[0])
             matrix = vstack([identity for _ in range(secondary_domain_size)])
             out = pybamm.Matrix(matrix) @ symbol
+        elif broadcast_type.startswith("tertiary"):
+            # Make copies of the child stacked on top of each other
+            identity = eye(symbol.shape[0])
+            matrix = vstack([identity for _ in range(tertiary_domain_size)])
+            out = pybamm.Matrix(matrix) @ symbol
         elif broadcast_type.startswith("full"):
             out = symbol * pybamm.Vector(np.ones(full_domain_size), domain=domain)
 
-        out.auxiliary_domains = auxiliary_domains
+        out.auxiliary_domains = auxiliary_domains.copy()
         return out
 
     def gradient(self, symbol, discretised_symbol, boundary_conditions):
@@ -444,7 +463,7 @@ class SpatialMethod:
             Discretised binary operator
 
         """
-        return bin_op.__class__(disc_left, disc_right)
+        return bin_op._binary_new_copy(disc_left, disc_right)
 
     def concatenation(self, disc_children):
         """Discrete concatenation object.
@@ -459,4 +478,4 @@ class SpatialMethod:
         :class:`pybamm.DomainConcatenation`
             Concatenation of the discretised children
         """
-        return pybamm.DomainConcatenation(disc_children, self.mesh)
+        return pybamm.domain_concatenation(disc_children, self.mesh)
